﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/topic",

// epi
    "epi/dependency"
],

function (
// dojo
    declare,
    lang,
    when,
    topic,

// epi
    dependency

) {

    // summary:
    //      Provides functionality for hiding and restoring side panels
    // tags:
    //      private, abstract

    return declare([], {

        // The profile obj 
        _profile: null,

        // The state of navigation bar
        _isNavigationVisible: null,

        // The state of tools bar
        _isToolsVisible: null,

        constructor: function () {
            // resolve the _profile dependency if not given 
            if (!this._profile) {
                this._profile = dependency.resolve("epi.shell.Profile");
            }

            // sets the sidepanels origional values on initlization
            this._setPanelSettings();
        },

        _setPanelSettings: function () {
            // summary: 
            //      Sets the visibility of navigation & toolsbar by retriving from respective panels
            // tags: 
            //    protected
            when(this._profile.get("navigation"), lang.hitch(this, function (navigationSetting) {
                this.isNavigationVisible = navigationSetting.visible;
            }));
            when(this._profile.get("tools"), lang.hitch(this, function (toolsSetting) {
                this.isToolsVisible = toolsSetting.visible;
            }));
        },

        _restoreSidePanels: function () {
            // summary: 
            //      Restores the side panels to their origional visbility
            // tags: 
            //      protected   
            topic.publish("/epi/layout/pinnable/navigation/toggle", this.isNavigationVisible);
            topic.publish("/epi/layout/pinnable/tools/toggle", this.isToolsVisible);
        },

        _hideSidePanels: function () {
            // summary: 
            //      hides the navigation and tools side bars
            // tags: 
            //      protected

            this._setPanelSettings();
            topic.publish("/epi/layout/pinnable/navigation/toggle", false);
            topic.publish("/epi/layout/pinnable/tools/toggle", false);
        }

    });

});