define([
    "dojo/_base/declare",
    "dojo/topic",
    // Resources
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.toolbar.buttons.compare",
    // Parent class and mixins
    "epi/shell/command/_Command",
    "dijit/Destroyable"
], function(
    declare,
    topic,
    // Resources
    localizations,
    // Parent class and mixins
    _Command,
    Destroyable
) {

    return declare([_Command, Destroyable], {
        // summary:
        //      A command that toggles between the compare view and the previous view when executed.
        //
        // tags:
        //      internal

        // label: [public] String
        //      The 'compare' text of the command to be used in visual elements.
        label: localizations.label,

        // iconClass: [public] String
        //      The CSS class which represents the icon to be used in visual elements.
        iconClass: "epi-iconCompare",

        // canExecute: [readonly] Boolean
        //      Flag which indicates whether this command is able to be executed.
        canExecute: true,

        // active: [readonly] Boolean
        //      Flag which indicates whether this command is in an active state.
        active: false,

        constructor: function() {
            var self = this;

            this.own(
                topic.subscribe("/epi/cms/action/switcheditmode", function() {
                    self.set("active", false);
                }),
                topic.subscribe("/epi/cms/action/eyetoggle", function(enabled) {
                    // Deactivate compare if the view settings are toggled on.
                    if (enabled) {
                        self.set("active", false);
                        topic.publish("/epi/shell/action/changeview/updatestate", { viewName: null });
                    }
                }),
                topic.subscribe("/epi/shell/action/viewchanged", function(type, args, data) {
                    // Deactivate compare if the view changes something other than compare.
                    var view = data && (data.viewName || (data.sender && data.sender.viewName));

                    self.set("active", (view === "sidebysidecompare"));

                    // Hide the compare command if the new view is a legacy view.
                    self.set("isAvailable", type === "epi-cms/contentediting/PageDataController");
                })
            );
        },

        _execute: function() {
            this.set("active", !this.active);
            if (this.active) {
                topic.publish("/epi/shell/action/changeview", "sidebysidecompare");
            } else {
                topic.publish("/epi/shell/action/changeview", null);
            }
        }
    });
});
