﻿define([
// Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-style",
    "dojo/Deferred",
    "dojo/on",
    "dojo/topic",
    "dojo/when",

//Dijit
    "dijit/registry",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_CssStateMixin",
    "dijit/_WidgetsInTemplateMixin",

// EPi Framework
    "epi/dependency",
    "epi/shell/dnd/Target",
    "epi/shell/command/_CommandProviderMixin",

//EPi CMS
    "./_ContentAreaTree",
    "./_ContentAreaTreeModel",
    "./_TextWithActionLinksMixin",
    "epi-cms/_ContentContextMixin",
    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/contentediting/viewmodel/ContentAreaViewModel",
    "epi/shell/widget/ContextMenu",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi-cms/widget/overlay/Block",
    "epi-cms/widget/command/CreateContentFromSelector",

    "epi-cms/widget/_HasChildDialogMixin",

    "epi-cms/contentediting/command/BlockRemove",
    "epi-cms/contentediting/command/BlockEdit",
    "epi-cms/contentediting/command/MoveToPrevious",
    "epi-cms/contentediting/command/MoveToNext",
    "epi-cms/contentediting/command/MoveOutsideGroup",
    "epi-cms/contentediting/command/Personalize",
    "epi-cms/contentediting/command/SelectDisplayOption",

// Resources
    "dojo/text!./templates/ContentAreaEditor.html",
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.editors.contentarea"
],
function (

// Dojo
    declare,
    lang,
    aspect,
    domStyle,
    Deferred,
    on,
    topic,
    when,

// Dijit
    registry,
    _WidgetBase,
    _TemplatedMixin,
    _CssStateMixin,
    _WidgetsInTemplateMixin,

// EPi Framework
    dependency,
    Target,
    _CommandProviderMixin,

// CMS
    _ContentAreaTree,
    _ContentAreaTreeModel,
    _TextWithActionLinksMixin,
    _ContentContextMixin,
    ContentActionSupport,
    ContentAreaViewModel,

    ContextMenu,
    _ValueRequiredMixin,
    BlockOverlay,
    CreateContentFromSelector,

    _HasChildDialogMixin,

    RemoveCommand,
    EditCommand,
    MoveToPrevious,
    MoveToNext,
    MoveOutsideGroup,
    Personalize,
    SelectDisplayOption,

// Resources
    template,
    resources
) {

    return declare([
        _WidgetBase,
        _TemplatedMixin,
        _WidgetsInTemplateMixin,
        _CssStateMixin,
        _ValueRequiredMixin,
        _ContentContextMixin,
        _CommandProviderMixin,
        _HasChildDialogMixin,
        _TextWithActionLinksMixin], {
        // summary:
        //      Editor for ContentArea to be able to edit my content area property in forms mode
        //      This should be a simple, non WYSIWYG listing of the inner content blocks with possibilities to add, remove and rearrange the content.
        //
        // tags:
        //      internal

        // baseClass: [public] String
        //    The widget's base CSS class.
        baseClass: "epi-content-area-editor",

        // res: Json object
        //      Language resource
        res: resources,

        // templateString: String
        //      UI template for content area editor
        templateString: template,

        // value: String
        //      Value of the content area
        value: null,

        // multiple: Boolean
        //  Value must be true, otherwise dijit.Form will trea the value as an object instead of an array
        multiple: true,

        // parent: Object
        //      Editor wrapper object containe the editor
        parent: null,

        // overlayItem: Object
        //      Source overlay of the content area in on page edit mode
        overlayItem: null,

        // model: Object
        //      Content area editor view model
        model: null,

        // intermediateChanges: Boolean
        //      Inherited from editor interface
        intermediateChanges: true,

        // editMode: String
        //      Flags to detect page edit mode (On page edit or Form edit mode or Create content mode)
        editMode: "onpageedit",

        // _contextService: [private] epi/shell/ContextService
        _contextService: null,

        // _preventOnBlur: [private] Boolean
        //      When set, the onBlur event is prevented.
        _preventOnBlur: false,

        _dndTarget: null,

        constructor: function() {
            this.allowedDndTypes = [];
        },

        onChange: function (value) {
            // summary:
            //    Called when the value in the widget changes.
            // tags:
            //    public callback
        },

        _handleModelChange: function (value) {
            // summary:
            //    Called when the value in the model changes.
            // tags:
            //    public callback

            this.validate();
            this.onChange(value);
        },

        _onBlur: function () {
            // summary:
            //      Override base to prevent the onBlur from being called when the _preventOnBlur flag is set.
            // tags:
            //      protected override

            if (this._preventOnBlur) {
                return;
            }
            this.inherited(arguments);
        },

        focus: function() {
            // summary:
            //    Focus the tree if there is a value, else focus the create block text.
            // tags:
            //    public
            if(this.model.get("value").length > 0) {
                this._focusManager.focus(this.treeNode);
            } else {
                this.textWithLinks.focus();
            }
        },

        postMixInProperties: function(){
            this.inherited(arguments);

            // NOTE: Check for this._commands to allow for mocking the commands without breaking _CommandProviderMixin.
            this.commands = this._commands || [
                new EditCommand(),
                new SelectDisplayOption(),
                new Personalize({ category: null }),
                new MoveOutsideGroup(),
                new MoveToPrevious(),
                new MoveToNext(),
                new RemoveCommand()
            ];

            //Create the view model
            this.model = new ContentAreaViewModel();
            this.own(this.model);

            this.own(this.model.watch("selectedItem", lang.hitch(this, function(name, oldValue, newValue){
                //Update the commands with the selected block
                this.updateCommandModel(newValue);
            })));

            this.own(on(this.model, "changed", lang.hitch(this, function() {

                if(!this._started || this._supressValueChanged){
                    return;
                }

                this._set("value", this.model.get("value"));

                //Call to the handle model change with the new value
                this._handleModelChange(this.value);
            })));
        },

        postCreate: function () {
            this.inherited(arguments);

            this.set("emptyMessage", resources.emptymessage);

            if (this.parent && this.overlayItem) {
                this.connect(this.parent, "onStartEdit", function() {
                    this._selectFromOverlay(this.overlayItem.model);
                });
            }

            this.own(topic.subscribe("/dnd/start", lang.hitch(this, this._startDrag)));
        },

        buildRendering: function () {
            this.inherited(arguments);

            this._contextService = this._contextService || dependency.resolve("epi.shell.ContextService");

            this.contextMenu = new ContextMenu();
            this.contextMenu.addProvider(this);

            this.own(this.contextMenu);

            this.setupActionLinks(this.actionsContainer);

             //Create the tree
            this.tree = new _ContentAreaTree({
                accept: this.allowedDndTypes,
                contextMenu: this.contextMenu,
                model: new _ContentAreaTreeModel({model: this.model})
            }, this.treeNode);

            this.own(this.tree);

            this.own(this._dndTarget = new Target(this.actionsContainer, {
                accept: this.allowedDndTypes,
                isSource: false,
                alwaysCopy: false,
                insertNodes: function(){}
            }));

            this.own(aspect.after(this._dndTarget, "onDropData", lang.hitch(this, function(dndData, source, nodes, copy) {
                var model = dndData[0].data;

                this.model.modify(lang.hitch(this, function() {
                    this.model.addChild(model);
                }));

            }), true));
        },

        startup: function () {
            this.inherited(arguments);

            this.tree.startup();
            this.contextMenu.startup();
        },

        isCreateLinkVisible: function () {
            // summary:
            //      Overidden mixin class, depend on currentMode will show/not create link
            // tags:
            //      protected
            var promise = new Deferred();
            var context = this._contextService && this._contextService.currentContext;

            //TODO: Move to the view model, duplicated in ContentArea. Don't dare changing at this point in time
            when(this.getCurrentContent(), function(contentData) {
                // Hide the create link if the content is not resourceable or we are in create mode.
                var allowedMode = true;
                if (context)
                {
                    allowedMode = context.capabilities.resourceable &&
                        !(context.currentMode === "create" || context.currentMode === "translate");
                }
                var hasCreatePermission = ContentActionSupport.isActionAvailable(contentData, ContentActionSupport.action.Create, ContentActionSupport.providerCapabilities.Create, true);
                promise.resolve(!!(allowedMode && hasCreatePermission));
            });

            return promise.promise;
        },

        executeAction: function (actionName) {
            // summary:
            //      Overridden mixin class executing click actions from textWithLinks widget
            // actionName: [String]
            //      Action name of link on content area
            // tags:
            //      public

            if (actionName === "createnewblock") {
                // HACK: Preventing the onBlur from being executed so the editor wrapper keeps this editor in editing state
                this._preventOnBlur = true;
                var command = new CreateContentFromSelector({
                    creatingTypeIdentifier: "episerver.core.blockdata",
                    createAsLocalAsset: true,
                    autoPublish: true
                });

                command.set("model", {
                    save: lang.hitch(this, function (block) {
                        this._preventOnBlur = false;
                        var value = lang.clone(this.get("value"), true) || [];
                        value.push(block);
                        this.set("value", value);
                        this.onChange(value);

                        // Now call onBlur since it's been prevented using the _preventOnBlur flag.
                        this.onBlur();
                    }),
                    cancel: lang.hitch(this, function () {
                        this._preventOnBlur = false;
                    })
                });
                command.execute();
            }
        },

        isValid: function () {
            // summary:
            //    Check if widget's value is valid.
            // isFocused:
            //    Indicate that the widget is being focused.
            // tags:
            //    protected
            return (!this.required || this.model.get("value").length > 0);
        },

         _setReadOnlyAttr: function (readOnly) {
            this._set("readOnly", readOnly);

            // hide actions when readonly
            domStyle.set(this.actionsContainer, "display", readOnly ? "none" : "");

            if (this._source) {
                this._source.isSource = !this.readOnly;
            }

            if(this.model) {
                this.model.set("readOnly", readOnly);
            }

            this.tree.set("readOnly", readOnly);
        },

        _checkAcceptance: function (source, nodes) {
            // summary:
            //      Customize checkAcceptance func
            // source: Object
            //      The source which provides items
            // nodes: Array
            //      The list of transferred items

            return this.readOnly ? false : this._source.defaultCheckAcceptance(source, nodes);
        },

        _setValueAttr: function (value) {

            this._set("value", value || []);

            //Reset the selection in the tree before we change the value on the model
            this.tree.set("selectedItems", null);

            this._supressValueChanged = true;
            this.model.set("value", value);
            this._supressValueChanged = false;
        },

        _selectFromOverlay: function(overlayModel) {
            var child = overlayModel && overlayModel.selectedItem && overlayModel.selectedItem.serialize(),
                model = this.model,
                path = ["root"];

            // exit if there is no overlay model to select item
            if(!child) {
                return;
            }

            if(child.contentGroup) {
                model = model.getChild({ name: child.contentGroup });
                path.push(model.id);
            }
            model = model.getChild(child);
            if(!model) {
                return;
            }

            path.push(model.id);

            model.set("selected", true);
            model.set("ensurePersonalization", overlayModel.selectedItem.ensurePersonalization);

            // TODO: move this selection into tree instead
            this.tree.set("path", path);

        },

        _startDrag: function (source, nodes, copy) {

            // close the editor when user start draging Block from BlockArea
            //TODO: This widget should not call a method on the parent
            var widget = registry.getEnclosingWidget(nodes[0]);
            if (widget.isInstanceOf(BlockOverlay) && this.parent.cancel) {
                // We set isModified to false (default value) because always synchronize value
                // between OPE and Editor
                this.parent.set("isModified", false);
                this.parent.cancel();
            }
        }
    });
});