define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-class",
    "dojo/dom-geometry",
    "dojo/on",
    "dojo/when",
    "dojo/dom-construct",

// dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/Button",
    "dijit/layout/_LayoutWidget",

// dgrid
    "dgrid/Keyboard",
    "dgrid/OnDemandList",
    "dgrid/Selection",

// epi
    "epi/dependency",
    "epi/shell/command/_WidgetCommandBinderMixin",
    "epi/shell/command/builder/MenuAssembler",
    "epi/shell/command/builder/ButtonBuilder",
    "epi/shell/dnd/Target",
    "epi/shell/dgrid/Formatter",
    "epi/shell/widget/_ModelBindingMixin",
    "epi/shell/widget/ContextMenu",
    "epi-cms/extension/events",
    "epi-cms/dgrid/formatters",
    "epi-cms/dgrid/WithContextMenu",
    "./ProjectComponentViewModel",
    "epi-cms/contentediting/ContentActionSupport",

// template
    "dojo/text!./ProjectComponent.html",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project",
    "epi/i18n!epi/nls/episerver.shared",

// Widgets in template
    "dijit/DropDownMenu",
    "dijit/form/DropDownButton",
    "dijit/Toolbar",
    "epi-cms/project/ProjectSelector",
    "epi-cms/project/PublishButtonLayout"
],

function (
// dojo
    declare,
    lang,
    aspect,
    domClass,
    domGeometry,
    on,
    when,
    domConstruct,
// dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Button,
    _LayoutWidget,

// dgrid
    Keyboard,
    OnDemandList,
    Selection,

// epi
    dependency,
    _WidgetCommandBinderMixin,
    MenuAssembler,
    ButtonBuilder,
    Target,
    Formatter,
    _ModelBindingMixin,
    ContextMenu,
    events,
    formatters,
    WithContextMenu,
    ProjectComponentViewModel,
    ContentActionSupport,

// template
    template,

// Resources
    res,
    sharedResources
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin, _WidgetCommandBinderMixin], {
        // summary:
        //
        // tags:
        //      internal

        model: null,

        commandProvider: null,

        templateString: template,

        res: res,

        modelBindingMap: {
            "projectItemQuery": ["projectItemQuery"],
            "projectItemSortOrder": ["projectItemSortOrder"],
            "selectedProject": ["selectedProject"],
            "created": ["created"],
            "createdBy": ["createdBy"],
            "createProjectInfo": ["createProjectInfo"],
            "placeholderState": ["placeholderState"],
            "publishFailed": ["publishFailed"],
            "projectStatus": ["projectStatus"]
        },

        _listClass: declare([OnDemandList, Formatter, Selection, Keyboard, WithContextMenu]),

        _list: null,

        _placeholderState: null,

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model) {
                this.own(
                    this.model = this.commandProvider = new ProjectComponentViewModel()
                );
            }
        },

        buildRendering: function () {
            this.inherited(arguments);

            this._setupList();
            this._setupProjectMenu();
            this._setupDnd();
        },

        startup: function () {

            var projectSelector = this.projectSelector,
                model = this.model;

            if (this._started) { return; }

            this.inherited(arguments);

            projectSelector.set("store", model.projectStore);

            model.initialize().then(lang.hitch(this, function () {

                this.own(
                    aspect.after(projectSelector, "onChange", lang.hitch(function (value) {
                        model.set("selectedProject", value);
                    }), true),
                    aspect.after(projectSelector, "onSortChanged", lang.hitch(function (value) {
                        model.set("projectSortOrder", value);
                    }), true),
                    // listen to view model for request to refresh the list
                    on(model, "refresh", lang.hitch(this, function () {
                        this._list.refresh({ keepScrollPosition: true });
                    })),
                    on(this.publishMenuNode, "click", lang.hitch(this, function () {
                        // Return focus back to the drop down button when the publish button is clicked. Otherwise
                        // focus will be reset back to the window when the publish button is hidden.
                        this.publishContainer.focus();
                    }))
                );

                this._list.startup();

            }));
        },

        layout: function () {
            this.inherited(arguments);

            this._resizeList();
        },

        _setupList: function () {

            var model = this.model,
                accessLevel = ContentActionSupport.accessLevel,
                contextMenu,
                list;

            function titleSelector(item) {
                if (ContentActionSupport.hasAccess(item.accessMask, accessLevel.Read) && item.path) {
                    return item.path.join(" > ");
                }

                return item.name;
            }

            function languageSelector(item) { 
                // If the current content language differs from the contentLanguage for the project item
                // return the content language for the project item
                if (item.contentLanguage !== model.contentLanguage) {
                    return { language: item.contentLanguage };
                } {
                    return null;
                }
            }

            contextMenu = new ContextMenu({ category: "itemContext" }); // Owned by the list
            list = new this._listClass({
                formatters: [
                    formatters.contentItemFactory(
                        "name", // nameSelector
                        titleSelector, // titleSelector
                        "typeIdentifier", // iconSelector
                        { hasMenu: true, settings: { title: sharedResources.action.options } }, // contextMenu settings
                        null, // thumbnailSelector
                        languageSelector // languageSelector
                    ),
                    this.model.statusFormatter // Adds status icon
                ],
                contextMenu: contextMenu,
                deselectOnRefresh: true,
                selectionMode: "single",
                selectionEvents: "click,dgrid-cellfocusin"
            }, this.itemListNode);

            this.own(
                this._list = list,
                list.on("dgrid-refresh-complete", lang.hitch(this, function (evt) {
                    when(evt.results.total, lang.hitch(this, function  (total) {
                        this.model.set("totalProjectItems", total);
                    }));
                })),
                list.on("dgrid-select", lang.hitch(this, function (evt) {
                    var item = evt.rows[0].data;
                    this.model.set("selectedProjectItem", item);
                })),
                list.on(".dgrid-row:dblclick", lang.hitch(this, function (evt) {
                    var item = this._list.row(evt).data;
                    if (ContentActionSupport.hasAccess(item.accessMask, accessLevel.Read)) {
                        this.model.requestContextChange(item.contentLink);
                    }
                })),
                list.on(on.selector(".dgrid-row", events.keys.del), lang.hitch(this, function () {
                    this.model.namedCommands.removeProjectItem.execute();
                }))
            );

            contextMenu.addProvider(this.model);
        },

        _resizeList: function () {
            var toolbarSize = domGeometry.getMarginBox(this.toolbar.domNode),
                size = lang.mixin({}, this._contentBox, { h: this._contentBox.h - toolbarSize.h });

            domGeometry.setMarginBox(this.itemListNode, size);
            this._list.resize();

            domGeometry.setMarginBox(this.placeholderNode, {
                l: 0,
                t: toolbarSize.h,
                h: size.h,
                w: size.w
            });
            domGeometry.setMarginBox(this.overlayNode, {
                l: 0,
                t: toolbarSize.h,
                h: size.h,
                w: size.w
            });
        },

        _setupProjectMenu: function () {

            var configuration = [
                    {
                        builder: new ButtonBuilder({ settings: { "class": "epi-fullWidthButton Sweet", isExecutingClass: "epi-loading" } }),
                        category: "publishmenu",
                        target: this.publishMenuNode
                    },
                    {
                        builder: new ButtonBuilder({ settings: { "class": "epi-flat epi-chromeless epi-chromeless--icon-only", showLabel: false } }),
                        category: "projectButton",
                        target: this.toolbarGroupNode
                    }
                ];

            this.own(
                new MenuAssembler({
                    configuration: configuration,
                    commandSource: this.model
                })
            );
        },

        _setupDnd: function () {
            var model = this.model,
                target;

            this.own(
                target = new Target(this.overlayNode, {
                    createItemOnDrop: false,
                    accept: ["episerver.core.icontentdata.reference"]
                }),
                aspect.after(target, "onDropData", function (dndItems) {
                    var items = dndItems.map(function (item) {
                        return item.data;
                    });
                    model.addProjectItems(items);
                }, true)
            );
        },

        _publishMenuClickHandler: function () {
            this.model.refreshProject();
        },

        _newProjectButtonClickHandler: function () {
            this.model.namedCommands.addProject.execute();
        },

        _setCreatedAttr: { node: "createdNode", type: "innerText" },

        _setCreatedByAttr: { node: "createdByNode", type: "innerText" },

        _setCreateProjectInfoAttr: { node: "createProjectInfoNode", type: "innerText" },

        _setProjectItemQueryAttr: function (value) {
            // Set store to null if query is null to stop list updates
            var store = value ? this.model.projectItemStore : null;
            this._list.set("store", store, value);
        },

        _setProjectItemSortOrderAttr: function (value) {
            this._list.set("sort", value);
        },

        _setSelectedProjectAttr: function (value) {
            this.projectSelector.set("value", value);
        },

        _setPublishFailedAttr: function (value) {
            this.publishFailNotificationTextNode.textContent = value;
            domClass.toggle(this.publishFailNotificationBarNode, "dijitHidden", !value);
        },

        _setProjectStatusAttr: function (value) {
            this.projectStatusMessageNode.textContent = value;
            domClass.toggle(this.projectStatusMessageNode, "dijitHidden", !value);
        },

        _setPlaceholderStateAttr: function (value) {

            function show(list) {
                list.forEach(function (node) { domClass.remove(node, "dijitHidden"); });
            }

            function hide(list) {
                list.forEach(function (node) {  domClass.add(node, "dijitHidden"); });
            }

            if (this.get("_placeholderState") === value) { return; }

            this._set("_placeholderState", value);

            // Toggle component elements visibility based on current placeholderState
            switch (value) {

                case "noProjects":
                    show([this.placeholderNode]);
                    hide([this.projectSelector.domNode, this.publishContainer.domNode, this.itemListNode, this.overlayNode]);

                    break;

                case "noSelectedProject":
                    show([this.projectSelector.domNode, this.publishContainer.domNode, this.placeholderNode]);
                    hide([this.itemListNode, this.overlayNode]);

                    break;

                case "noProjectItems":
                    show([this.projectSelector.domNode, this.publishContainer.domNode, this.itemListNode, this.overlayNode]);
                    hide([this.placeholderNode]);

                    break;

                case "projectPublished":
                    show([this.projectSelector.domNode, this.publishContainer.domNode, this.itemListNode]);
                    hide([this.placeholderNode, this.overlayNode]);

                    break;

                default:
                    show([this.projectSelector.domNode, this.publishContainer.domNode, this.itemListNode, this.overlayNode]);
                    hide([this.placeholderNode]);

            }

            domClass.toggle(this.overlayNode, "projectVisible", value === "noProjectItems");

        }
    });
});
