define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/Evented",
    "dojo/Stateful",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/promise/all",
    "dojo/when",

// dijit
    "dijit/Destroyable",

//epi
    "epi/epi",
    "epi/datetime",
    "epi/dependency",
    "epi/username",
    "epi-cms/core/ContentReference",
    "epi/shell/xhr/ErrorHandler",
    "epi-cms/contentediting/ContentActionSupport",

// Commands
    "epi/shell/command/withConfirmation",
    "./command/AddProject",
    "./command/PublishProject",
    "./command/ReadyToPublishProjectItem",
    "./command/RefreshProjectItems",
    "./command/RemoveProject",
    "./command/RemoveProjectItem",
    "./command/RenameProject",
    "./command/SortProjectItems",

// Resources
    "epi/i18n!epi/nls/episerver.cms.components.project",
    "epi/i18n!epi/nls/episerver.shared.action",
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.versionstatus"
],

function (
// dojo
    declare,
    lang,
    Deferred,
    Evented,
    Stateful,
    topic,
    domClass,
    all,
    when,

// dijit
    Destroyable,

// epi
    epi,
    epiDate,
    dependency,
    username,
    ContentReference,
    ErrorHandler,
    ContentActionSupport,

// Commands
    withConfirmation,
    AddProject,
    PublishProject,
    ReadyToPublishProjectItem,
    RefreshProjectItems,
    RemoveProject,
    RemoveProjectItem,
    RenameProject,
    SortProjectItems,

// Resources
    res,
    resShared,
    versionStatusRes
) {
    return declare([Stateful, Destroyable, Evented], {
        // summary:
        //      The view model for the epi-cms/project/ProjectComponent
        // tags:
        //      internal

        // commands: [readonly] epi/shell/command/_Command[]
        //      Commands to be consumed by the view.
        commands: null,

        // namedCommands: [readonly] Object
        //      Way to access commands using named keys.
        namedCommands: null,

        // created: [readonly] string
        //      A view prepared version of the date the selected project was created.
        created: "",

        // createdBy: [readonly] string
        //      A view prepared version of the user that created the selected project.
        createdBy: "",

        // profile: [readonly] Profile
        //      The current user profile
        profile: null,

        // createdProjectInfo: [readonly] string
        //      A view prepared version of the placeholder info.
        createProjectInfo: "",

        // contentLanguage: [readonly] string
        //      The current UI language
        contentLanguage: "",

        // totalProjectItems: [readonly] int
        //      Current project items count.
        totalProjectItems: -1,
        
        // hasProject: [public] bool
        //      True if we have any projects
        hasProject: false,

        // projectStore: [readonly] Store
        //      A REST store for interacting with projects.
        projectStore: null,

        // projectSortOrder: [public] Object
        //      The order projects are sorted by.
        projectSortOrder: [{ attribute: "created", descending: true }],

        // projectItemStore: [readonly] Store
        //      A REST store for interacting with project items.
        projectItemStore: null,

        // projectItemQuery: Query
        //      Query object holding parameters to get a projects items.
        projectItemQuery: null,

        // projectItemSortOrder: [public] Object
        //      The order to sort project items by.
        projectItemSortOrder: null,

        // projectStatus: [public] string
        //      State that indicates if a project and it's contents are available.
        projectStatus: "",

        // publishFailed: [public] string
        //      The message to show if publishing of a project has failed.
        publishFailed: "",

        // selectedProject: [public] Object
        //      The project that is currently selected.
        selectedProject: null,

        // selectedProjectItem: [public] Object
        //      The project item that is currently selected.
        selectedProjectItem: null,

        // placeholderState: [public] string
        //      State that indicates if a project and it's contents are available.
        placeholderState: null,

        _selectedProjectProfileKey: "epi.selected-project-id",

        _sortOrderProfileKey: "epi.project-sort-order",

        postscript: function () {
            this.inherited(arguments);

            // Resolve the stores from the registry if they haven't been injected. This allows
            // us to mock the stores when testing.
            this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
            this.projectItemStore = this.projectItemStore || dependency.resolve("epi.storeregistry").get("epi.cms.project.item");
            this.profile = this.profile || dependency.resolve("epi.shell.Profile");

            this._createCommands();
        },

        initialize: function () {
            // summary:
            //      
            // returns: Promise
            //      
            // tags:
            //      public

            return all(
                // Set the sort order based on what is stored in the profile, fall back to default value if nothing stored.
                when(this.profile.get(this._sortOrderProfileKey), lang.hitch(this, function (sort)  {
                    var option = this._findSortOption("key", sort) || this.namedCommands.sortProjectItems.options[0]; // default to name ascending
                    this._changeAttrValue("projectItemSortOrder", option.value); // use the internal setter to not trigger a profile save
                })),
                // Load the selected project from the profile and update hasProject.
                when(this._loadSelectedProjectFromProfile(), lang.hitch(this, function (project) {

                    // if a project has been resolved we know that there is at least one available
                    // so that we can pass it along to the hasProject check
                    var hasProject = !!project || undefined;

                    this._updateSelectedProjectDependencies(project);
                    this._changeAttrValue("selectedProject", project);  // use the internal setter to not trigger a profile save

                    return this._updateHasProject(hasProject);
                })),
                // Get the current content language from the profile
                when(this.profile.getContentLanguage(), lang.hitch(this, function (contentLanguage) {
                    this.set("contentLanguage", contentLanguage);
                }))
            );
        },

        getCommands: function () {
            // summary:
            //      Returns all available commands
            // tags:
            //      public

            return this.commands;
        },

        addProject: function (project) {
            // summary:
            //      Adds a project and sets it as the selectedProject
            // tags:
            //      public

            this.projectStore.add(project)
                .then(lang.hitch(this, function (value) { this.set("selectedProject", value); }))
                .otherwise(ErrorHandler.forXhr);
        },

        updateProject: function (project) {
            // summary:
            //      Adds a project and sets it as the selectedProject
            // tags:
            //      public

            this.projectStore.put(project)
                .then(lang.hitch(this, "set", "selectedProject"))
                .otherwise(ErrorHandler.forXhr);
        },

        publishProject: function (selectedProjectId) {
            // summary:
            //      Starts a publishing task for a project and polls continously to verify the results
            // tags:
            //      public

            var def = new Deferred(),
                store = this.projectStore,
                self = this;

            if (isNaN(selectedProjectId)) {
                throw new Error("selectedProjectId is not a number");
            }

            store.executeMethod("Publish", selectedProjectId).then(
                function (updatedProject) {
                    // Call notify on the store once the publish method is complete to
                    // ensure any observes are kept up to date.
                    store.notify(updatedProject, store.getIdentity(updatedProject));

                    self.set("selectedProject", updatedProject);
                    def.resolve(updatedProject);
                },
                function () {
                    def.reject("publishFailed");
                }
            );

            return def.promise;
        },

        refreshProject: function () {
            // summary:
            //      Refreshes the selected project, particularly information if the project can be published or not.
            // tags:
            //      public

            var self = this;

            if (!this.selectedProject) {
                this._updateHasProject();
                this.emit("refresh");
                return;
            }

            return this.projectStore.refresh(this.selectedProject.id).then(
                function (value) {
                    self.set("selectedProject", value);
                    self.emit("refresh");
                },
                function (ex) {
                    self.set("selectedProject", null);
                }
            );
        },

        addProjectItems: function (contentReferences, projectId) {
            // summary:
            //      Adds the given content references as items of the project.
            // tags:
            //      public

            // Ensure the content reference we received are valid.
            contentReferences = contentReferences.filter(function (reference) {
                if (ContentReference.isContentReference(reference)) {
                    return true;
                }
                console.log("The given value '" + reference + "' is not a valid content reference.");
            });

            if (!projectId && !this.selectedProject) {
                throw new Error("If no projectId is provided a selectedProject must be set in the context");
            }

            projectId = projectId || this.get("selectedProject").id;

            var store = this.projectItemStore,
                promises = contentReferences.map(function (reference) {
                    return store.add({ contentLink: reference, projectId: projectId });
                });

            return all(promises)
                .then(lang.hitch(this, "emit", "refresh"))
                .otherwise(ErrorHandler.forXhr);
        },

        removeSelectedProjectItem: function () {
            // summary:
            //      Removes a project item from a project.
            // tags:
            //      public

            this.projectItemStore.remove(this.selectedProjectItem.id)
                .then(lang.hitch(this, function () {
                    this.set("totalProjectItems", (this.totalProjectItems - 1));
                }))
                .otherwise(ErrorHandler.forXhr);
        },

        markProjectItemAsReadyToPublish: function () {
            // summary:
            //      Mark the project item as ready to publish
            // returns: Promise
            //      A promise that resolves when the refresh project has completed.
            // tags:
            //      public

            var promise = this.projectItemStore.executeMethod("ReadyToPublish", this.selectedProjectItem.id);

            promise
                .then(lang.hitch(this, "refreshProject"))
                .otherwise(ErrorHandler.forXhr);

            return promise;
        },

        requestContextChange: function (contentReference) {
            // summary:
            //      Requests that the UI changes the context to the specified contentReference.
            // tags:
            //      public

            topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + contentReference }, { sender: this });
        },

        statusFormatter: function (value, object, node, options) {
            // summary:
            //      Usable for the view to get an icon indicating the status of an item.
            // tags:
            //      public

            var accessLevel = ContentActionSupport.accessLevel,
                versionStatus = [
                    "",                                 // NotCreated
                    "Stop epi-icon--danger",            // Rejected
                    "Pen",                              // CheckedOut
                    "Checkmark epi-icon--success",      // CheckedIn
                    "Published epi-icon--success",      // Published
                    "PreviouslyPublished",              // PreviouslyPublished
                    "Clock",                            // DelayedPublish
                    "Clock epi-icon--danger"            // Expired
                ],
                cssClass = "epi-icon" + versionStatus[object.status],
                title = ContentActionSupport.versionLocalizations[object.status] || "";

            // if the content item is deleted set the deleted class on the entire node
            if (object.isDeleted) {
                cssClass = "epi-iconWarning epi-icon--warning";
                title = versionStatusRes.versionnotfound;
                domClass.add(node, "epi-content--deleted");
            } else if (!ContentActionSupport.hasAccess(object.accessMask, accessLevel.Read)) {
                cssClass = "epi-iconLock";
                title = res.status.message.noreadaccess;
            } else if (!ContentActionSupport.hasAccess(object.accessMask, accessLevel.Publish)) {
                cssClass = "epi-iconWarning epi-icon--warning";
                title = res.status.message.nopublishaccess;
            }

            return '<span class="dijitReset dijitInline dijitIcon epi-objectIcon ' + cssClass + '" title="' + title + '"></span>' + value;
        },

        _createCommands: function () {

            var commandArgs = { model: this },
                refreshProjectItems,
                addProject,
                publishProject,
                readyToPublishProjectItem,
                removeProject,
                removeProjectItem,
                renameProject,
                sortProjectItems = new SortProjectItems(commandArgs);

            this.own(
                refreshProjectItems = new RefreshProjectItems(commandArgs),
                addProject = new AddProject(commandArgs),
                publishProject = new PublishProject(commandArgs),
                readyToPublishProjectItem = new ReadyToPublishProjectItem(commandArgs),
                removeProject = withConfirmation(new RemoveProject({ model: this, store: this.projectStore }), null, {
                    title: res.command.removeproject.label,
                    description: res.command.removeproject.confirmation,
                    confirmActionText: resShared.deletelabel,
                    cancelActionText: resShared.cancel
                }),
                removeProjectItem = withConfirmation(new RemoveProjectItem({ model: this, store: this.projectItemStore }), null, {
                    title: res.command.removeprojectitem.label,
                    description: res.command.removeprojectitem.confirmation,
                    confirmActionText: resShared.remove,
                    cancelActionText: resShared.cancel
                }),
                renameProject = new RenameProject(commandArgs)
            );

            this.set("commands", [
                refreshProjectItems,
                addProject,
                renameProject,
                publishProject,
                removeProject,
                readyToPublishProjectItem,
                removeProjectItem,
                sortProjectItems
            ]);

            this.set("namedCommands", {
                addProject: addProject,
                refreshProjectItems: refreshProjectItems,
                publishProject: publishProject,
                removeProject: removeProject,
                removeProjectItem: removeProjectItem,
                sortProjectItems: sortProjectItems
            });
        },

        _selectNextProject: function () {
            // summary:
            //      Selects next project
            // tags:
            //      private

            when(this._getProjects(0, 1, this.get("projectSortOrder")), lang.hitch(this, function (results) {
                var value = results.length > 0 ? results[0]: null;
                this.set("selectedProject", value);
                this.set("hasProject", results.length > 0);
            }));
        },

        _getProjects: function (start, count, sort) {
            // summary:
            //      Gets projects based on attributes
            // tags:
            //      private
            return this.projectStore.query(null, { start: start, count: count, sort: sort });
        },

        _updateHasProject: function (hasProject) {
            // summary:
            //      updates hasProject boolean depending on getHasProject result
            // tags:
            //      private

            return when(hasProject === true ? [hasProject] : this._getProjects(0, 1), lang.hitch(this, function (results) {
                var result = results.length > 0;
                this.set("hasProject", result);

                return new Deferred().resolve(result).promise;
            }));
        },

        _loadSelectedProjectFromProfile: function () {
            // summary:
            //      Load the selected project from the user profile
            // returns: Promise
            //      The project that was stored in the profile
            // tags:
            //      internal

            var projectStore = this.projectStore;

            //Load the selected project from the profile
            return when(this.profile.get(this._selectedProjectProfileKey), function (projectId) {
                if (!projectId) {
                    return null;
                }

                // Need to get the project from the store in order to ensure it still exists.
                // Otherwise return null
                return when(projectStore.get(projectId)).otherwise(function (ex) { return null; });
            });
        },

        _findSortOption: function (key, term) {

            var options = this.namedCommands.sortProjectItems.options,
                option = null;

            term && options.some(function (item) {
                if (term === item[key]) {
                    option = item;
                    return true;
                }
            });

            return option;
        },

        _updateSelectedProjectDependencies: function (selectedProject) {

            var created = "",
                createdBy = "",
                publishFailed = "",
                projectStatus = "";

            // Refresh the list query when the selected project changes.
            this.set("projectItemQuery", selectedProject ? { projectId: selectedProject.id } : null);

            if (selectedProject) {
                created = epiDate.toUserFriendlyString(epiDate.deserialize(selectedProject.created));
                createdBy = username.toUserFriendlyString(selectedProject.createdBy, null, true);

                if (selectedProject.status === "publishfailed" && selectedProject.failedItemsCount > 0) {
                    projectStatus = res.status.message.publishfailed;
                    // update the notification bar text
                    publishFailed = selectedProject.failedItemsCount > 1 ? res.notifications.publishfailed.notificationtextmultipleitems
                                                                         : res.notifications.publishfailed.notificationtextsingleitem;
                    publishFailed = lang.replace(publishFailed, { failedItemsCount: selectedProject.failedItemsCount });
                } else if (selectedProject.status === "active" || (selectedProject.status === "publishfailed" && selectedProject.failedItemsCount === 0)) {
                    projectStatus = res.status.message.publishfailed;
                } else {
                    projectStatus = res.status.message[selectedProject.status] || "";
                }
            }

            this.set("created", created);
            this.set("createdBy", createdBy);
            this.set("publishFailed", publishFailed);
            this.set("projectStatus", projectStatus);
        },

        _updatePlaceholderState: function () {

            var selectedProject = this.get("selectedProject"),
                hasSelectedProject = !!selectedProject,
                hasProjects = this.get("hasProject"),
                hasProjectItems = this.get("totalProjectItems") > 0,
                state = "";

            if (hasSelectedProject && selectedProject.status === "published" ) {
                state = "projectPublished";
            }
            else if (hasSelectedProject && hasProjects && hasProjectItems) {
                // should set placeholderState to 'ready' when there are projects available, projectitems, and a project is selected
                state = "ready";

            } else if (!hasSelectedProject && hasProjects) {
                // should set placeholderState to 'noSelectedProject' when there are projects available, no projectitems, and no project is selected
                state = "noSelectedProject";
                this.set("createProjectInfo", res.placeholder.noprojectselected);

            } else if (hasProjects && !hasProjectItems) {
                // should set placeholderState to 'noProjectItems' when there are projects available, no projectitems, and a project is selected
                state = "noProjectItems";

            } else if (!hasProjects) {
                // should set placeholderState to 'noProjects' when there are no projects, no projectitems, and no project is selected
                state = "noProjects";
                this.set("createProjectInfo", res.placeholder.noprojectscreated);
            }

            this.set("placeholderState", state);
        },

        _hasProjectSetter: function (value) {
            this.hasProject = value;
            this._updatePlaceholderState();
        },

        _totalProjectItemsSetter: function (value) {
            this.totalProjectItems = value;
            this._updatePlaceholderState();
        },

        _selectedProjectSetter: function (selectedProject) {

            // Early exit if the project has not changed
            if (epi.areEqual(this.selectedProject, selectedProject)) { return; }

            this.selectedProject = selectedProject;
            this._updateSelectedProjectDependencies(selectedProject);

            !this.hasProject && selectedProject && this.set("hasProject", true);
            !selectedProject && this._selectNextProject();

            // Store the selected project.
            this.profile.set(this._selectedProjectProfileKey, selectedProject && selectedProject.id, { location: "server" });

        },

        _projectItemSortOrderSetter: function (sortOrder) {
            // summary:
            //      Store the selected project in the user profile

            var option,
                key;

            if (sortOrder && sortOrder === this.projectItemSortOrder) { return; }

            this.projectItemSortOrder = sortOrder;

            option = this._findSortOption("value", sortOrder);
            key = option && option.key || null;

            this.profile.set(this._sortOrderProfileKey, key, { location: "server" });

        }
    });
});