define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojox/html/entities",
// epi
    "epi/dependency",
    "epi-cms/contentediting/_ContentEditingNotification",

    // template
    "dojo/text!./ProjectNotification.html",

// resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.notifications"
],

function (
// dojo
    declare,
    lang,
    when,
    entities,
// epi
    dependency,
    _ContentEditingNotification,

//template
    template,

// resources
    resources
) {

    return declare([_ContentEditingNotification], {
        // summary:
        //      Show notification if the content is part of a project
        // tags:
        //      internal

        // projectService: [readonly] ProjectService
        //      A service for interacting with projects
        projectService: null,

        postscript: function() {
            this.inherited(arguments);

            this.projectService = this.projectService || dependency.resolve("epi.cms.ProjectService");
        },

        _onExecuteSuccess: function(content) {
            if(!content || !content.contentLink) {
                return;
            }

            var self = this;

            when(this.projectService.getProjectsForContent(content.contentLink), function(projects) {
                if(projects.length === 0) {
                    self._setNotification(null);
                    return;
                }

                var projectNames = projects.map(function(project) {
                    return entities.encode(project.name);
                }).join(", ");

                var content = lang.replace(template, {
                    partOfProject: resources.partofproject,
                    projects: projectNames
                });

                self._setNotification({
                   content: content
                });
            });
        }
    });
});
