define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-construct",
    "dojo/on",

    "dojox/html/entities",

    "epi",
    // Resources
    "dojo/text!dijit/form/templates/Select.html",
    "epi/i18n!epi/shell/ui/nls/episerver.shared.header",
    // Grid
    "./ProjectSelectorGrid",

    // Parent class and mixins
    "dijit/form/_FormValueWidget",
    "epi/shell/widget/DeferredDropDownButton"
],
function (
    declare,
    lang,
    aspect,
    domConstruct,
    on,
    entities,

    epi,
    // Resources
    template,
    localizations,
    // Grid
    ProjectSelectorGrid,

    // Parent class and mixins
    _FormValueWidget,
    DeferredDropDownButton
) {
    return declare([DeferredDropDownButton, _FormValueWidget], {
        // summary:
        //      Drop down button for selecting projects
        // tags:
        //      internal

        // class: [protected] String
        //      class of the button
        "class": "epi-project-gadget__project-selector epi-chromeless epi-chromeless--with-arrow epi-flat",

        iconClass: "epi-iconProject",

        value: null,

        // store: [public] Store
        //      A store to query for the list of options.
        store: null,

        // sort: [public] Object
        //      How the projects are sorted in the drop down.
        sort: null,

        // query: [public] Object
        //      A query to use when fetching items from the store.
        query: null,

        buildRendering: function () {
            this.inherited(arguments);

            this.own(
                this.dropDown = new ProjectSelectorGrid(),
                on(this.dropDown, "change", lang.hitch(this, function (e) {
                    this.set("value", e.value, true);
                    this.closeDropDown(true);
                }), true),
                on(this.dropDown, "sortchange", lang.hitch(this, function (e) {
                    this.onSortChanged(e.value);
                }), true)
            );
        },

        compare: function (value1, value2) {
            // summary:
            //      Compare two values (as returned by get("value") for this widget).
            // tags:
            //      protected
            return epi.areEqual(value1, value2) ? 0 : -1;
        },

        loadAndOpenDropDown: function () {
            // summary:
            //      Opens the drop down and refreshes the content.
            // returns: Promise
            //      Promise for the drop down widget that resolves when
            //      the drop down is created and loaded.
            // tags:
            //      protected

            if (this.isLoaded()) {
                // If the drop down has already been loaded then we need to do a refresh after opening.
                return this.inherited(arguments)
                    .then(lang.hitch(this, function() {
                        return this.dropDown.refresh();
                    }))
                    .then(lang.hitch(this, function() {
                        this.dropDown.focus(this.value);
                    }));
            }

            return this.inherited(arguments);
        },

        onSortChanged: function (sort) { },

        _getQueryAttr: function () {
            return this.query || {};
        },

        _setQueryAttr: function (value) {
            this._set("query", value);
            if (this.dropDown) {
                this.dropDown.set("query", value);
            }
        },

        _setStoreAttr: function (store) {
            this._set("store", store);
            if (this.dropDown) {
                this.dropDown.set("store", store);
            }
        },

        _setValueAttr: function (value, internal) {
            this.set("label", entities.encode((value && value.name) || ""));
            !internal && this.dropDown && this.dropDown.set("selectedProject", value);

            this.inherited(arguments);
        }
    });
});
