﻿define([
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/lang",
    "dojo/keys",
    "dojo/when",
    "epi",
    // Resources
    "dojo/text!./ProjectSelectorGrid.html",
    "epi/i18n!epi/shell/ui/nls/episerver.shared.header",
    "epi/i18n!epi/cms/nls/episerver.cms.components.project",
    // Grid
    "dgrid/OnDemandGrid",
    "dgrid/extensions/DijitRegistry",
    "dgrid/Keyboard",
    "dgrid/Selection",
    "epi/shell/dgrid/Focusable",
    "epi/shell/dgrid/selection/Exclusive",
    "epi/shell/dgrid/util/misc",
    "epi-cms/dgrid/formatters",
    // Parent class and mixins
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_KeyNavContainer",
    "epi/shell/widget/_FocusableMixin"
], function(
    declare,
    event,
    lang,
    keys,
    when,
    epi,
    // Resources
    template,
    localizations,
    res,
    // Grid
    OnDemandGrid,
    DijitRegistry,
    Keyboard,
    Selection,
    Focusable,
    Exclusive,
    misc,
    formatters,
    // Parent class and mixins
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _KeyNavContainer,
    _FocusableMixin
) {

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _KeyNavContainer, _FocusableMixin], {
        // summary:
        //      Sortable project selector
        //
        // tags:
        //      internal

        // baseClass: [protected] String
        //      Base CSS class of the widget, used to construct CSS classes to indicate widget state.
        baseClass: "epi-project-selector-grid",

        // templateString: [protected] String
        //      A string that represents the widget template.
        templateString: template,

        // store: [public] Store
        //      The store that will be queried for projects.
        store: null,

        // query: [public] Object
        //      A query to use when fetching items from the store.
        query: null,

        // markedContentLink: [public] String
        //      The content link of the version which should be marked as in use.
        selectedProject: null,

        postMixInProperties: function() {
            // summary:
            //      Called after the parameters to the widget have been read-in,
            //      but before the widget template is instantiated. The column
            //      definitions are setup at this time.
            // tags:
            //      protected
            this.inherited(arguments);

            this.columns = {
                id: {
                    className: "epi-grid--5",
                    label: "",
                    formatter: function() {
                        return misc.icon("epi-radio-icon--secondary");
                    }
                },
                name: {
                    className: "epi-grid--30",
                    label: localizations.name
                },
                status: {
                    className: "epi-grid--20",
                    label: localizations.status,
                    formatter: function (state) {
                        return res.status.state[state.toLowerCase()];
                    }
                },
                created: {
                    className: "epi-grid--25",
                    label: localizations.created,
                    formatter: formatters.localizedDate
                },
                createdBy: {
                    className: "epi-grid--20",
                    label: localizations.by,
                    formatter: formatters.userName,
                    sortable: false
                }
            };
        },

        buildRendering: function() {
            // summary:
            //      Construct the UI for this widget via dijit/_TemplatedMixin with the addition
            //      of a dgrid added via code.
            // tags:
            //      protected
            this.inherited(arguments);

            this.grid = new (declare([OnDemandGrid, DijitRegistry, Selection, Keyboard, Focusable, Exclusive]))({
                columns: this.columns,
                cellNavigation: false,
                sort: [{ attribute: "created", descending: true }],
                className: "epi-project-grid-select epi-grid-select epi-plain-grid epi-focused",
                cleanEmptyObservers: false,
                deselectOnRefresh: false,
                selectionMode: "exclusive",
                selectionEvents: "click",
                query: this.get("query")
            }, this.gridNode);

            // Own the grid and the event handlers.
            this.own(
                this.grid,
                this.grid.on("dgrid-select", lang.hitch(this, "_projectSelected")),
                this.grid.on("dgrid-refresh-complete", lang.hitch(this, "_refreshComplete")),
                this.grid.addKeyHandler(keys.ENTER, lang.hitch(this, "_gridEnterKeypress")),
                this.grid.on("dgrid-sort", lang.hitch(this, "_sortChanged"))
            );
        },

        startup: function() {
            // summary:
            //      Processing after the widget and its children have been created and
            //      added to the DOM. Setup the initial query and store for the grid.
            // tags:
            //      protected
            if (this._started) {
                return;
            }
            this.inherited(arguments);

            this.grid.startup();
            this.grid.set("store", this.store);
        },

        resize: function() {
            // summary:
            //      Resize the version selector and its children.
            // tags:
            //      public
            this.inherited(arguments);
            this.grid.resize();
        },

        refresh: function() {
            // summary:
            //      Refreshes the version selector to ensure the data displayed is up to date.
            // tags:
            //      public

            if (!this._started) {
                return;
            }

            return this.grid.refresh().then(
                lang.hitch(this, function (projects) {
                    var selectedProject = this.get("selectedProject"),
                        projectExists = selectedProject && projects.some(function(item) {
                            if(item.id === selectedProject.id){
                                selectedProject = item;
                                return true;
                            }
                            return false;
                        }, this);

                    this.set("selectedProject", projectExists ? selectedProject : null);
                })
            );
        },

        _projectSelected: function (e) {
            // summary:
            //      Extracts the version information and emits a change event.
            // tags:
            //      private

            var value = e.rows && e.rows[0].data;

            if (value && !epi.areEqual(value, this.get("selectedProject"))) {
                this.set("selectedProject", value);
                this.emit("change", { value: value });
            }
        },

        _refreshComplete: function(evt) {
            // summary:
            //      Selects the current version and marks the marked version when a refresh occurs.
            // tags:
            //      private

            when(evt.results.total, lang.hitch(this, function(total) {
                this.emit("loaded");
            }));
        },

        _gridEnterKeypress: function(e) {
            // summary:
            //      Select the given row when the enter key is pressed.
            // tags:
            //      private

            // Clear the selection first. This ensures we don't end up with multiple items selected.
            this.grid.clearSelection();
            this.grid.select(e);
        },

        _sortChanged: function (evt) {
            this.emit("sortchange", { value: evt.sort });
        },

        _setSelectedProjectAttr: function(selectedProject) {
            // summary:
            //      Sets the selected project and ensures the UI is in the correct state.
            // tags:
            //      protected
            this._set("selectedProject", selectedProject);

            // Clear the value regardless if there is a value or not. This ensures we don't
            // end up with multiple items selected.
            this.grid.clearSelection();
            if (selectedProject) {
                this.grid.select(selectedProject.id);
            }
        }
    });
});
