define([
// dojo
    "dojo/_base/declare",
    "dojo/when",
    "dojo/promise/all",
// epi
    "epi/dependency"
],

function (
// dojo
    declare,
    when,
    all,
// epi
    dependency
) {

    return declare(null, {
        // summary:
        //      A service for interacting with projects
        // tags:
        //      internal

        // projectItemStore: [readonly] Store
        //      A REST store for interacting with project items.
        projectItemStore: null,

        // projectStore: [readonly] Store
        //      A REST store for interacting with projects.
        projectStore: null,

        constructor: function(params) {
            declare.safeMixin(this, params);

            this.projectItemStore = this.projectItemStore || dependency.resolve("epi.storeregistry").get("epi.cms.project.item");
            this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
        },

        getProjectItemsForContent: function(contentLink) {
              // summary:
            //      Get the associated projects items for the given content link
            //
            // contentLink: String
            //      The content link to get the project items for for
            // tags:
            //      internal

             return this.projectItemStore.query({contentLink: contentLink});
        },

        getProjectsForContent: function(contentLink) {
            // summary:
            //      Get the associated projects for the given content link
            //
            // contentLink: String
            //      The content link to get the projects for
            // tags:
            //      internal

            var self = this;

            return when(this.getProjectItemsForContent(contentLink), function(projectItems) {
                // If no project items where found return an empty array
                if(projectItems.length === 0) {
                    return [];
                }

                return self.getProjects(projectItems);
            });
        },

        getProjects: function(projectItems) {
             // summary:
            //      Get the projects for the project items
            //
            // projectItems: int[]
            //      The project items to get the projects for
            // tags:
            //      internal

            var projectStore = this.projectStore;
            var projectPromises = projectItems.map(function(projectItem) {
                return projectStore.get(projectItem.projectId);
            });

            return all(projectPromises);
        }
    });
});
