﻿define([
    "dojo/_base/declare",
// Parent class
    "epi-cms/project/command/_ProjectCommand",
// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.command.publishproject"
], function (
    declare,
// Parent class
    _ProjectCommand,
// Resources
    res
) {

    return declare([_ProjectCommand], {
        // tags:
        //      internal

        // category: [readonly] String
        //      A category which provides a hint about how the command could be displayed.
        category: "publishmenu",

        // label: [public] String
        //      The action text of the command to be used in visual elements.
        label: res.label,

        // isExecuting: [public] Boolean
        //      Indicates wheter the command is currently executing.
        isExecuting: false,

        _execute: function () {
            // summary:
            //      Executes this command assuming canExecute has been checked.
            // tags:
            //      protected

            var model = this.model,
                self = this;

            self.set("isExecuting", true);
            self.set("label", res.labelexecuting);

            model.publishProject(this.model.selectedProject.id).then(
                function() {
                    self.set("isExecuting", false);
                    self.set("label", res.labelexecuted);
                },
                function() {
                    self.set("isExecuting", false);
                    self.set("label", res.label);
                    console.error("Failed to publish project: ", model.selectedProject.name);
            });
        },

        _setCanExecute: function () {
            var selectedProject = this.model.selectedProject,
                status = selectedProject && selectedProject.status;

            // The command shouldn't be executable if a project is not selected or cannot be published.
            this.set("canExecute", !!selectedProject && selectedProject.canBePublished);
            this.set("isAvailable", status != "Published");

            if (selectedProject) {
                var label = status == "Published" ? res.labelexecuted : status == "Publishing" ? res.labelexecuting : res.label;

                this.set("label", label);
            }
        }
    });
});
