define([
    "dojo/_base/declare",
    "epi-cms/contentediting/ContentActionSupport",
    // Resources
    "epi/i18n!epi/nls/episerver.cms.components.project.command.readytopublishprojectitem",
    // Parent class
    "epi-cms/project/command/_ProjectCommand"
], function(
    declare,
    ContentActionSupport,
    // Resources
    localizations,
    // Parent class
    _ProjectCommand
) {

    return declare([_ProjectCommand], {
        // tags:
        //      internal
        
        // category: [readonly] String
        //      A category which provides a hint about how the command could be displayed.
        category: "itemContext",

        // label: [public] String
        //      The action text of the command to be used in visual elements.
        label: localizations.label,

        // modelPropertyToWatch: [private] String
        //      The property on the model to watch for changes.
        modelPropertyToWatch: "selectedProjectItem",

        _execute: function() {
            // summary:
            //      Executes this command assuming canExecute has been checked.
            // tags:
            //      protected
            return this.model.markProjectItemAsReadyToPublish();
        },

        _setCanExecute: function() {
            // summary:
            //      Sets canExecute based on the state of the model.
            // tags:
            //      protected
            var item = this.model.selectedProjectItem,
                hasAccess = !!item && ContentActionSupport.hasAccess(item.accessMask, ContentActionSupport.accessLevel.Edit),
                status = ContentActionSupport.versionStatus;

            this.set("canExecute", !!item && (item.status === status.CheckedOut || item.status === status.Rejected) && item.isDeleted === false);
            this.set("isAvailable", hasAccess);
        }
    });
});
