﻿define([
// dojo
    "dojo",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/_base/Deferred",
// epi
    "epi",
    "epi/dependency",
    "epi-cms/widget/_ItemSelectorBase",
    "epi-cms/widget/_DropDownWidgetItemMixin",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.channelselector"],
function (
// dojo
    dojo,
    lang,
    array,
    Deferred,
// epi
    epi,
    dependency,
    _ItemSelectorBase,
    _DropDownWidgetItemMixin,
// resources
    res
) {
    return dojo.declare([_ItemSelectorBase, _DropDownWidgetItemMixin], {
        // tags:
        //      internal

        // defaultItem: [public] Object
        //    The widget's default item to be added to the menu.
        defaultItem: {
            name: res.defaultitem,
            resolutionName: res.defaultitem
        },

        // _isFinished: [private] dojo.Deferred
        //    Deferred to indicate when widget finished its setup process.
        _isFinished: null,

        // _channelHashMap: [private] Object
        //     Map the channel display name (localized) and its name.
        _channelHashMap: {},

        postMixInProperties: function () {
            // summary:
            //    Sets up the data store
            //
            // tags:
            //    protected
            this.inherited(arguments);
            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("epi.cms.displaychannels");
        },

        postCreate: function () {
            // summary:
            //    Raised after the widget creating, sets the header and
            //    load the items to be added to the menu.
            //
            // tags:
            //    protected
            this._isFinished = new Deferred();
            this.inherited(arguments);
            this.set("header", res.header);
            dojo.when(this._store.query({ query: "getallregistered" }), lang.hitch(this, function (registeredChannels) {
                this._createHashMap(registeredChannels);
                this.checkedItem = (this.checkedItem ? this._channelHashMap[this.checkedItem] : this.defaultItem.name);
                registeredChannels.unshift(this.defaultItem);
                this.set("items", registeredChannels);
                this._isFinished.resolve({ status: true });
            }));
        },

        isSetupFinished: function () {
            // summary:
            //    Return a deferred indicating when the widget finished
            //    its setup process.
            //
            // tags:
            //    public
            return this._isFinished;
        },

        _createHashMap: function (items) {
            // summary:
            //    Creates a map with the channel display name and its name.
            array.forEach(items, lang.hitch(this, function (item) {
                this._channelHashMap[item.channelName] = item.name;
            }));
        }
    });
});