﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/_base/connect",
    "dojo/_base/Deferred",
    "dojo/dom-class",
// dijit
    "dijit/DropDownMenu",
    "dijit/form/DropDownButton",
    "dijit/focus",
// epi
    "epi",
    "epi/dependency",
    "epi-cms/widget/ChannelSelector",
    "epi-cms/widget/ResolutionSelector",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.channelbutton"],
function (
// dojo
    declare,
    lang,
    array,
    connect,
    Deferred,
    domClass,
// dijit
    DropDownMenu,
    DropDownButton,
    focusUtil,
// epi
    epi,
    dependency,
    ChannelSelector,
    ResolutionSelector,
    resources) {
    return declare([DropDownButton], {
        // tags:
        //      internal

        // _contextMenu: [private] dijit.DropDownMenu
        //    The context menu for the button.
        _contextMenu: null,

        // baseClass: [public] String
        //    CSS base classes for the button.
        baseClass: "epi-mediumButton epi-chromelessButton",

        // iconClass: [public] String
        //    CSS class that adds the button icon.
        iconClass: "epi-iconDevice",

        // contextMenuProperties [public] Object
        //    Context menu properties.
        contextMenuProperties: {},

        // _channelSelector: [private] Object
        //    the channel selector widget.
        _channelSelector: null,

        // _resolutionSelector: [private] Object
        //    the resolution selector widget.
        _resolutionSelector: null,

        // _viewSettingsManager: [private] Object
        //    The view manager
        _viewSettingsManager: null,

        postMixInProperties: function () {
            // summary:
            //    Mixin the widget properties.
            //
            // tags:
            //    protected
            this.inherited(arguments);

            lang.mixin(this.contextMenuProperties, {
                "class": "epi-DropDownMenu"
            });
        },

        postCreate: function () {
            // summary:
            //    Raised after the widget is created. It will create
            //    the widget's context menu and add its children.
            //
            // tags:
            //    protected
            this.inherited(arguments);

            this._viewSettingsManager = dependency.resolve("epi.viewsettingsmanager");

            var viewSettingResolution = this._viewSettingsManager.getSettingProperty("resolution");
            var viewSettingChannel = this._viewSettingsManager.getSettingProperty("epichannel");

            if (!this.dropDown) {

                this.dropDown = dijit.DropDownMenu(this.contextMenuProperties);

                this._channelSelector = new ChannelSelector({ checkedItem: viewSettingChannel });
                connect.connect(this._channelSelector, "onItemSelected", this, this._onChannelSelected);

                this._resolutionSelector = new ResolutionSelector({ checkedItem: viewSettingResolution });
                connect.connect(this._resolutionSelector, "onItemSelected", this, this._onResolutionSelected);

                this._addChildren(this._channelSelector);
                this._addChildren(this._resolutionSelector);

                Deferred.when(this._channelSelector.isSetupFinished(), lang.hitch(this, function (result) {
                    this._activeButton(!this._isDefaultSelection());
                    this._updateTooltip();

                    this._setupKeyboardNav();
                }));
            }
        },

        _setupKeyboardNav: function () {
            /// summary: Setup to support keyboard navigation to select chanel

            // put menu items into an array to proccess
            var focusItems = [];
            array.forEach([this._channelSelector, this._resolutionSelector], function (menu) {
                focusItems = focusItems.concat(
                    array.map(menu.menuNode.getChildren(), function (item) {
                        return {
                            menuItem: item,
                            menu: menu.menuNode
                        };
                    })
                );
            });

            // handler for focusNext, focusPrev action
            var focusItemIndex = 0;
            var focusHandler = lang.hitch(this, function (action) {
                focusItemIndex += action === "next" ? 1 : -1;
                focusItemIndex = focusItemIndex >= focusItems.length ? 0 : focusItemIndex;
                focusItemIndex = focusItemIndex < 0 ? focusItems.length - 1 : focusItemIndex;

                var focusItem = focusItems[focusItemIndex];
                focusItem.menu.focusChild(focusItem.menuItem);
            });
            var focusNextHandler = function () { focusHandler("next"); };
            var focusPrevHandler = function () { focusHandler("prev"); };

            // Connects
            connect.connect(this.dropDown, "onOpen", this, function (e) {
                focusUtil.focus(this._channelSelector.menuNode.domNode);
            });
            this._channelSelector.menuNode.focusNext = this._resolutionSelector.menuNode.focusNext = focusNextHandler;
            this._channelSelector.menuNode.focusPrev = this._resolutionSelector.menuNode.focusPrev = focusPrevHandler;
        },


        _updateTooltip: function () {

            var tooltiptext = null;

            if (this._isDefaultSelection()) {
                tooltiptext = resources.defaulttooltip;
            }
            else {
                var selectedChannel = this._channelSelector.currentSelectedItem;
                var selectedResolution = this._resolutionSelector.currentSelectedItem;
                tooltiptext = lang.replace(resources.tooltip, {
                    channel: selectedChannel.name,
                    resolution: selectedResolution.name
                });
            }
            this.set("title", tooltiptext);
        },

        _isDefaultSelection: function () {
            // summary:
            //    Checks if the selected items on both widgets are the default items.
            //
            // tags:
            //    private
            return (this._channelSelector.isDefaultItemSelected() && this._resolutionSelector.isDefaultItemSelected());
        },

        _onChannelSelected: function (/*Object*/channel) {
            // summary:
            //    Raised when a channel is selected.
            //
            // channel:
            //   The selected channel
            //
            // tags:
            //    private event
            var defaultResolution = this._resolutionSelector.getDefaultResolution(channel);

            // if the current resolution is diferent from the channel's default resolution
            // update the resolution selector widget.
            if (!this._resolutionSelector.currentSelectedItem ||
               (this._resolutionSelector.currentSelectedItem && this._resolutionSelector.currentSelectedItem.name != defaultResolution.name)) {
                this._resolutionSelector.checkItem(defaultResolution);
            }

            this._changeView(defaultResolution, channel);

        },

        _onResolutionSelected: function (/*Object*/resolution) {
            // summary:
            //    Raised when a resolution is selected.
            //
            // resolution:
            //   The selected resolution
            //
            // tags:
            //    private event
            this._changeView(resolution);

        },

        _changeView: function (/*Object*/resolution, /*Object*/channel) {
            // summary:
            //    Change the view to the given resolution.
            //
            // resolution:
            //    Resolution information
            //
            // channel:
            //    Channel information.
            //
            // tags:
            //    private

            var messageData = [],
                value;

            if (resolution) {
                value = resolution.name === this._resolutionSelector.defaultItem.name ?
                    null :
                    resolution.width + "x" + resolution.height;

                messageData.push({ key: "resolution", value: value });
            }

            if (channel) {
                value = channel.name === this._channelSelector.defaultItem.name ?
                    null :
                    channel.channelName;

                messageData.push({ key: "epichannel", value: value });
            }

            if (messageData.length) {
                connect.publish("/epi/cms/action/viewsettingvaluechanged", [messageData]);
            }


            this._activeButton(!this._isDefaultSelection());
            this._updateTooltip();
        },

        _addChildren: function (/*Object*/item) {
            // summary:
            //    Populate the context menu with the configured items.
            //
            // item:
            //    The item that will be added to the dropdown
            //
            // tags:
            //    private
            this.dropDown.addChild(item);
        },

        _activeButton: function (/*Boolean*/active) {
            // summary:
            //      Set active state for button widget
            //
            // active:
            //    States if the button will be checked or not.
            //
            // tags:
            //      private
            if (!this.dropDown) { return; }
            if (active) {
                domClass.add(this.domNode, "dijitChecked");
            } else {
                domClass.remove(this.domNode, "dijitChecked");
            }
        }
    });

});