﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-class",
    "dojo/dom-construct",

    "dojo/aspect",
    "dojo/Evented",
// dgrid
    "dgrid/Keyboard",
    "dgrid/OnDemandList",
    "dgrid/Selection",
// dojox
    "dojox/html/entities",
// epi
    "epi",

    "epi/shell/dgrid/Formatter",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/core/ContentReference",

    "epi-cms/dgrid/formatters",
    "epi-cms/dgrid/DnD",
    "epi-cms/dgrid/WithContextMenu",

    "epi-cms/widget/ReadOnlyContentList"
],

function (
// dojo
    declare,
    lang,

    domClass,
    domConstruct,

    aspect,
    Evented,
// dgrid
    Keyboard,
    OnDemandList,
    Selection,
// dojox
    htmlEntities,
// epi
    epi,

    Formatter,
    TypeDescriptorManager,

    ContentReference,

    formatters,
    DnD,
    WithContextMenu,

    ReadOnlyContentList
) {

    return declare([ReadOnlyContentList, Evented], {
        // summary:
        //      Base class for lists for different content.
        // description:
        //      This is a base class that can be used to quickly create a list.
        // tags:
        //      public

        // _gridClass: [private] Object
        //      Content grid class declaration
        _gridClass: declare([OnDemandList, Formatter, Selection, DnD, Keyboard, WithContextMenu]),

        // resources: [public] Object
        //      Public resources for grid content list
        resources: "",

        // noDataMessage: [public] String
        //      No data message text for grid content list incase it has no data to display
        noDataMessage: null,

        // editingItem: [public] dojo.data.Item
        //      Keeps the selected content information
        editingItem: null,

        postCreate: function () {

            this.inherited(arguments);

            domClass.add(this.domNode, "epi-contentList");
        },

        _setCurrentItemAttr: function (/*dojo.data.Item*/value) {
            // summary:
            //      Store the selected content information on the first load
            // value: [dojo.data.Item]
            // tags:
            //      protected

            if (!this.get("editingItem")) {
                this.set("editingItem", value);
            }

            this._set("currentItem", value);
            this.select(value);
        },

        _setNoDataMessageAttr: function (/*String*/value) {
            // summary:
            //      Set text message for grid content list incase it has no data.
            // value: [String]
            //      No data message text
            // tags:
            //      protected

            this.noDataMessage = value;
            this.grid.set("noDataMessage", value);
        },

        _dndNodeCreator: function (/*dojo.data.Item*/item, hint) {
            // summary:
            //      Custom DnD avatar creator method
            // item: [dojo.data.Item]
            // hint: [String]
            // tags:
            //      private

            var dndTypes = TypeDescriptorManager.getAndConcatenateValues(item.typeIdentifier, "dndTypes");
            if (!dndTypes && this.dndTypes) {
                dndTypes = this.dndTypes;
            }

            return {
                "node": domConstruct.create("div").appendChild(document.createTextNode(item.name)),
                "type": dndTypes,
                "data": item
            };
        },

        getListSettings: function () {
            // summary:
            //      Initialization a list.
            // tags:
            //      protected, extension

            var menu = {
                hasMenu: !!this.contextMenu,
                settings: {
                    title: epi.resources.action.options
                }
            };

            // Init content list
            return lang.delegate(this._getBaseSettings(menu), {
                noDataMessage: this.noDataMessage,
                dndParams: {
                    copyOnly: true,
                    selfAccept: false,
                    accept: [],
                    creator: lang.hitch(this, this._dndNodeCreator)
                }
            });
        },

        setupEvents: function () {
            // summary:
            //      Initialization events on list.
            // tags:
            //      protected

            this.inherited(arguments);

            this.own(
                // Connect to create new item link when there's no children for the current parent.
                this.grid.on(".epi-createContent:click", lang.hitch(this, function (e) {
                    this.emit("createItemAction", {});
                })),

                aspect.around(this.grid, "insertRow", lang.hitch(this, this._aroundInsertRow))
            );
        },

        _onSelect: function (/*Event*/evt) {
            // summary:
            //      Emit selected event
            // evt: [Event]
            //      Select event of the grid
            // tags:
            //      private

            this.inherited(arguments);

            if (this.grid.selectionMode === "single") {
                this.emit("itemSelected", evt.rows[0].data);
            }
        },

        editContent: function (/*dojo.data.Item*/item) {
            // summary:
            //      Enable editing mode for the given content
            // item: [dojo.data.Item]
            //      Content that wanted to edit
            // tags:
            //      public

            // Do nothing if we do not found the given content in the grid
            if (!item || !this.grid.row(item).element) {
                return;
            }

            this.emit("itemAction", item);

            // After the selected content loaded, save its information
            this.set("editingItem", this.currentItem);
        },

        _onChangeContext: function (evt) {
            // summary:
            //      Go to content edit mode when change context event raised
            // evt: [Event]
            //      The click event that caused the change context.
            // tags:
            //      public

            this.editContent(this.grid.row(evt).data);
        },

        toggleCut: function (/*dojo.data.Item*/item, /*Boolean*/isCut) {
            // summary:
            //      Toggle cutting styles for the given item in the grid content list
            // item: [dojo.data.Item]
            // isCut: [Boolean]
            // tags:
            //      public

            var row;
            if (item) {
                row = this.grid.row(ContentReference.toContentReference(item.contentLink).id);
                if (row && row.element) {
                    domClass.toggle(row.element, "epi-opacity50", isCut);
                }
            }
        }

    });

});