﻿define([
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/dom-style",
    "dojo/dom-construct",
    "dojo/string",
    "dojo/when",

// dijit
    "dijit/focus",
    "dijit/form/Button",

// epi.shell
    "epi/dependency",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Dialog",

// epi.cms
    "epi-cms/core/ContentReference",
    "epi-cms/core/PermanentLinkHelper",
    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/widget/ContentSelectorDialog",
    "epi-cms/widget/_SelectorBase",

// resources

    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselector"
],

function (
// dojo
    array,
    declare,
    lang,
    domClass,
    domStyle,
    domConstruct,
    stringUtil,
    when,

    focusManager,
    Button,

// epi.shell
    dependency,
    TypeDescriptorManager,
    Dialog,

// epi.cms
    ContentReference,
    PermanentLinkHelper,
    ContentActionSupport,
    ContentSelectorDialog,
     _SelectorBase,

// resources

    localization
) {

    return declare([_SelectorBase], {
        // tags:
        //      internal

        roots: null,

        value: null,

        // required: Boolean
        //		User is required to enter data into this field.
        required: false,

        // missingMessage: [public] String
        //    Message which is displayed when required is true and value is empty.
        missingMessage: localization.requiredmessage,

        // tooltipPosition: String[]
        //		See description of `dijit.Tooltip.defaultPosition` for details on this parameter.
        tooltipPosition: [],

        // contentLink: [pubic] String
        //      Content reference of the content being edited.
        contentLink: null,

        permanentLink: null,

        // canSelectOwnerContent: [public] Boolean
        //      Indicates whether the owner content that is being edited can be selected
        canSelectOwnerContent: false,

        // accessLevel: [public] Enum
        //      The access right level for the content node can be selected
        accessLevel: ContentActionSupport.accessLevel.Read,

        // showAllLanguages: Boolean
        //      Flags to indicate that the content tree should show all content in multiple languages or not.
        showAllLanguages: true,

        _setDisabledAttr: function (value) {
            this.inherited(arguments);
            this.button.set("disabled", value);
            this._set("disabled", value);
        },

        buildRendering: function () {
            // summary:
            //		Creates and append the input hidden field to the dom to preserve the contentLink id
            // tags:
            //    protected

            var inputName = this.name ? (this.name.replace(/"/g, "&quot;")) : '';
            this.input = domConstruct.create("input", { type: "hidden", name: inputName });

            this.inherited(arguments);
            this.domNode.appendChild(this.input);
        },

        postMixInProperties: function () {
            // summary:
            //		Initialize properties
            // tags:
            //    protected

            this.inherited(arguments);

            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("epi.cms.content.light");

            if (!this.roots) {
                var contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");
                var settings = contentRepositoryDescriptors[this.repositoryKey];
                this.roots = settings.roots;
            }
        },

        postCreate: function () {
            // summary:
            //      Overridden to initialize ui elements when no value is set.

            this.inherited(arguments);

            if (!this.value) {
                this._updateDisplayNode(null);
            }
        },

        _setReadOnlyAttr: function (value) {
            this._set("readOnly", value);

            domStyle.set(this.button.domNode, "display", value ? "none" : "");
            domStyle.set(this.clearButton, "display", value ? "none" : "");
            domClass.toggle(this.domNode, "dijitReadOnly", value);
        },

        _setValueAttr: function (value) {
            //summary:
            //    Value's setter.
            //
            // value: String
            //    Value to be set.
            //
            // tags:
            //    protected

            this._setValueAndFireOnChange(value);
        },

        _getDialog: function () {
            // summary:
            //		Create page tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node existings or not
            if (this.dialog && this.dialog.domNode) {
                return this.dialog;
            }

            var title = localization.title;
            if (this.allowedTypes && this.allowedTypes.length === 1) {
                var name = TypeDescriptorManager.getResourceValue(this.allowedTypes[0], "name");
                if (name) {
                    title = stringUtil.substitute(localization.format, [name]);
                }
            }

            this.contentSelectorDialog = new ContentSelectorDialog({
                canSelectOwnerContent: this.canSelectOwnerContent,
                showButtons: false,
                roots: this.roots,
                allowedTypes: this.allowedTypes,
                showAllLanguages: this.showAllLanguages
            });

            this.dialog = new Dialog({
                title: title,
                dialogClass: "epi-dialog-portrait",
                content: this.contentSelectorDialog,
                destroyOnHide: true
            });

            this.connect(this.contentSelectorDialog, "onChange", "_setDialogButtonState");
            this.connect(this.dialog, 'onExecute', '_onDialogExecute');
            this.connect(this.dialog, 'onHide', '_onDialogHide');

            return this.dialog;
        },

        _isContentAllowed: function (contentTypeIdentifier) {
            // Summary:
            //    Checks whether the given content type belongs to the any of the allowedTypes/parent type or not
            //
            // contentTypeIdentifier: String
            //    The content type identifier
            //
            // tags:
            //    Private

            var allowed = array.indexOf(this.allowedTypes, contentTypeIdentifier) !== -1;

            if (!allowed) {
                //If it's not a direct match, check if the type inherits an accepted type.
                allowed = array.some(this.allowedTypes, function (type) {
                    return TypeDescriptorManager.isBaseTypeIdentifier(contentTypeIdentifier, type);
                });
            }

            return allowed;
        },

        _onButtonClick: function () {
            // Summary:
            //    Handle pick button click
            // tags:
            //    private

            when(this._getContentData(this.contentLink), lang.hitch(this, function (content) {

                // if the current content is any of allowedTypes then allow to select it
                this.canSelectOwnerContent = content && this._isContentAllowed(content.typeIdentifier);
                var dialog = this._getDialog();
                this.isShowingChildDialog = true;

                var valueAsContentReference;
                if (this.value) {
                    valueAsContentReference = this._convertValueToContentReference(this.value);
                }

                if (valueAsContentReference) {
                    valueAsContentReference = valueAsContentReference.createVersionUnspecificReference().toString();
                }

                this.contentSelectorDialog.set("value", valueAsContentReference);
                dialog.show();
            }));
        },

        _onDialogExecute: function () {
            //summary:
            //    Handle dialog close
            // tags:
            //    private

            var value = this.contentSelectorDialog.get('value');
            this._setValueAndFireOnChange(value);
        },

        _convertValueToContentReference: function (value) {
            //summary:
            //
            // tags:
            //    private
            return new ContentReference(value);
        },

        _setValueAndFireOnChange: function (value) {
            //summary:
            //    Gets the content data by selected value, updates the view and calls onChange if value was changed
            // tags:
            //    private

            var contentLink = value === '-' ? this.contentLink : value;

            this.set("permanentLink", null);

            when(this._getContentData(contentLink), lang.hitch(this, function (content) {

                var hasChange = this.value !== contentLink;

                this.value = contentLink;
                this.input.value = this.value;
                this._started && this.validate();
                this._updateDisplayNode(content);

                if (hasChange) {
                    this.onChange(contentLink);
                }

                if (content) {
                    this.set("permanentLink", PermanentLinkHelper.toPermanentLink(content));
                }
            }));
        },

        _getContentData: function (contentLink) {
            //summary:
            //    Loads the content data from the store
            // tags:
            //    private

            if (!contentLink) {
                return null;
            }
            return this._store.get(contentLink);
        },

        _onFocus: function () {
            // summary:
            //		This is where widgets do processing for when they start being active,
            //		such as changing CSS classes.  See onFocus() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },


        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },

        focus: function () {
            // summary:
            //       Put focus on this widget
            // tags:
            //      public

            if (!this.button.disabled && this.button.focusNode && this.button.focusNode.focus) {
                try {
                    focusManager.focus(this.button.focusNode);
                } catch (e) {
                    /*squelch errors from hidden nodes*/
                }
            }
        },

        _updateDisplayNode: function (content) {
            //summary:
            //    Update widget's display text
            // tags:
            //    protected

            this.inherited(arguments);

            // conent selector also sets Link id 
            if (content) {
                this.set("selectedContentLink", new ContentReference(content.contentLink).id);
            }
            else {
                this.set("selectedContentName", localization.helptext);
                this.set("selectedContentLink", "");
            }
        },

        reset: function () {
            this.set("value", null);
        },

        _setDialogButtonState: function (contentLink) {
            // summary:
            //    Set state of dialog button.
            //
            // tags:
            //    protected

            var self = this;

            if (!contentLink) {
                return;
            }

            when(self._store.get(contentLink), function (content) {
                self.dialog.definitionConsumer.setItemProperty(self.dialog._okButtonName, "disabled", !ContentActionSupport.hasAccess(content.accessMask, self.accessLevel));
            });
        }
    });
});