﻿define([
// Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",

// Dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetBase",

// EPi
    "epi",
    "epi/shell/TypeDescriptorManager",

    "epi-cms/ApplicationSettings",
    "epi-cms/widget/ContentTree",
    "epi-cms/widget/ContextualContentForestStoreModel",

// EPi Framework
    "epi/shell/widget/_ActionProviderWidget",
    "epi/dependency",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselectordialog"
],

function (
// Dojo
    array,
    declare,
    lang,
    when,

// Dijit
    _TemplatedMixin,
    _Widget,

// EPi
    epi,
    TypeDescriptorManager,

    ApplicationSettings,
    ContentTree,
    ContextualContentForestStoreModel,

// EPi Framework
    _ActionProviderWidget,
    dependency,

// Resources
    localization
) {

    return declare([_Widget, _TemplatedMixin, _ActionProviderWidget], {
        // summary:
        //    Content selector widget.
        //
        // description:
        //    Used for editing PropertyPage properties in flyout editor.
        //
        // tags:
        //    internal xproduct

        // _contentRef: [private] epi.cms.core.ContentReference
        //    Reference to the currently selected content.
        _contentRef: null,

        // res: [private] Object
        //    Resource bundle.
        res: localization,

        // roots: [public] Array
        //    A list of references to the root contents.
        roots: null,

        // typeIdentifiers: [public] Array | String
        //    Type identifiers to filter.
        typeIdentifiers: null,

        showButtons: true,

        model: null,

        // canSelectOwnerContent: [private] Boolean
        //    Indicates whether select current owner content button should be available.
        canSelectOwnerContent: true,

        // multiRootsMode: [public] Boolean
        //      Indicate that multiple roots enabled in the tree or not
        multiRootsMode: false,

        // templateString: [protected] String
        //    Widget's template string.
        templateString: '<div><div data-dojo-attach-point="_treeNode"></div></div>',

        _typesToDisplay: null,

        // showAllLanguages: Boolean
        //      Flags to indicate that the content tree should show all content in multiple languages or not.
        showAllLanguages: false,

        // selectedContentType: String
        //      The content type which is selected. (used when restore content for instance)
        selectedContentType: null,

        buildRendering: function () {
            this.inherited(arguments);

            var typeIdentifiers = this._getTypesToDisplay(),
                model = this.model,
                roots = this.roots || (model && model.roots);

            this.preventContextualContentFor = this.preventContextualContentFor || this._getSettingFromContentType();
            if (!model || this.showContextualContent) {
                model = new ContextualContentForestStoreModel({
                    roots: roots,
                    typeIdentifiers: typeIdentifiers,
                    notSupportContextualContents: this.preventContextualContentFor,
                    showAllLanguages: this.showAllLanguages
                });
            }

            model.set("view", "contentselector");

            this.tree = new ContentTree({
                roots: roots,
                typeIdentifiers: typeIdentifiers,
                allowManipulation: false,
                model: model,
                expandExtraNodeItems: ApplicationSettings.startPage // Set extra node items to expand when the given tree loaded
            }, this._treeNode);

            this.connect(this.tree, "onClick", "_onTreeNodeClick");
        },

        _getSettingFromContentType: function () {
            // summary:
            //      Get preventContextualContentFor settings form selected content type
            // tags:
            //      private

            if (!this.selectedContentType) {
                return null;
            }
            var contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");

            var repositoryDescriptor,
                matchType = function (type) {
                    return TypeDescriptorManager.isBaseTypeIdentifier(this.selectedContentType, type);
                };
            for (var index in contentRepositoryDescriptors) {
                var descriptor = contentRepositoryDescriptors[index];
                if (array.some(descriptor.containedTypes, matchType, this)) {
                    repositoryDescriptor = descriptor;
                    break;
                }
            }

            return repositoryDescriptor.preventContextualContentFor;
        },
        
        _getTypesToDisplay: function () {
            if (!this._typesToDisplay) {

                var typesToDisplay = [];
                this._getContainerTypesRecursive(this.allowedTypes, typesToDisplay);
                this._typesToDisplay = typesToDisplay;
            }

            return this._typesToDisplay;
        },

        _getContainerTypesRecursive: function (types, results, checkedTypes) {
            if (!checkedTypes) {
                checkedTypes = [];
            }

            array.forEach(types, lang.hitch(this, function (type) {
                // To avoid infinite recursion, check if this type is already processed
                if (array.indexOf(checkedTypes, type) === -1) {
                    // Mark as checked
                    checkedTypes.push(type);

                    // Add the type itself if it isn't added already
                    if (array.indexOf(results, type) === -1) {
                        results.push(type);
                    }

                    // If there are any container types, process them as well
                    var containerTypesForType = TypeDescriptorManager.getValue(type, "containerTypes");

                    if (containerTypesForType) {
                        this._getContainerTypesRecursive(containerTypesForType, results, checkedTypes);
                    }
                }
            }));
        },

        _setValueAttr: function (value) {
            //summary:
            //    Value's setter.
            //
            // value: String
            //    Value to be set.
            //
            // tags:
            //    protected

            //construct ContentReference object
            if (value) {
                this._contentRef = value;
                var contentLinkToSelect = this._contentRef === '-' ? null : this._contentRef;

                if (this.multiRootsMode) {
                    // We need to wait until tree load top level children and then fire onLoad() event
                    when(this.tree.onLoadDeferred, lang.hitch(this, function() {
                        this.tree.selectContent(contentLinkToSelect);
                    }));
                } else {
                    if (!epi.isEmpty(contentLinkToSelect)) {
                        this.tree.selectContent(contentLinkToSelect);
                    }
                }
            }
            else {
                this._contentRef = null;
                this._clearSelectedContent();
            }
        },

        _getValueAttr: function () {
            //summary:
            //    Value's getter
            // tags:
            //    protected

            this.inherited(arguments);

            if (this._contentRef) {
                return this._contentRef.toString();
            }
            else {
                return '';
            }
        },

        _clearSelectedContent: function () {
            // summary:
            //      Clear selected node(s) in tree
            // tags:
            //      private

            this.tree.set("selectedNodes", []);
            this.tree.set("selectedItems", []);
        },

        getActions: function () {
            // summary:
            //      Overridden from _ActionProvider to get the select current owner content action added to the containing widget
            //
            // returns:
            //      An array contining a select page action definition, if it is not a shared block

            return this.canSelectOwnerContent ? [
                {
                    name: "selectpage",
                    label: localization.currentpage,
                    settings: { type: "submit" },
                    action: lang.hitch(this, function () {
                        this._setValueOwnerContent();
                    })
                }
            ]
            : [];
        },

        _onTreeNodeClick: function (content) {
            // summary:
            //    Handle the inner tree's content change event.
            //
            // content: Object
            //    The content object.
            //
            // tags:
            //    private

            var value = content.contentLink;

            var acceptedType = array.indexOf(this.allowedTypes, content.typeIdentifier) !== -1;

            if (!acceptedType) {
                //If it's not a direct match, check if the type inherits an accepted type.
                acceptedType = array.some(this.allowedTypes, function (type) {
                    return TypeDescriptorManager.isBaseTypeIdentifier(content.typeIdentifier, type);
                });
            }

            if(!acceptedType){
                this.tree.lastFocused.setSelected(false);
                value = null;
            }

            this.set('value', value);
            this.onChange(this.get('value'));
        },

        _setValueOwnerContent: function () {
            this.set('value', '-');
        },

        onChange: function (contentLink) {
            // summary:
            //    Fired when value is changed.
            //
            // contentLink:
            //    The content link
            // tags:
            //    public, callback
        },

        focus: function () {
            this.tree.focus();
        }

    });

});