﻿define([
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-geometry",
    "dojo/dom-style",
    "dojo/on",
    "dojo/when",

// dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",
    "dijit/layout/BorderContainer", // Used in template

// dojox
    "dojox/html/entities",

// epi
    "epi/string",
    "epi/shell/command/_WidgetCommandBinderMixin",
    "epi/shell/widget/_FocusableMixin",
    "epi/shell/widget/_ModelBindingMixin",
    "epi/shell/widget/dialog/Alert",

    "epi/shell/widget/ContextMenu",
    "epi-cms/widget/SearchBox",
    "epi-cms/widget/FolderTree",
    "epi-cms/widget/ContentList",
    "epi-cms/widget/FilesUploadDropZone",
    "epi-cms/widget/UploadUtil",
    "epi-cms/widget/viewmodel/HierarchicalListViewModel",

// template
    "dojo/text!./templates/HierarchicalList.html",
    "dojo/text!./templates/AssetsDropZone.html",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.hierachicallist"
],

function (
// dojo
    array,
    declare,
    lang,
    aspect,
    domClass,
    domConstruct,
    domGeometry,
    domStyle,
    on,
    when,

// dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,
    BorderContainer,

//dojox
    htmlEntities,

// epi
    epistring,

    _WidgetCommandBinderMixin,
    _FocusableMixin,
    _ModelBindingMixin,
    Alert,

    ContextMenu,
    SearchBox,
    FolderTree,
    ContentList,
    DropZone,
    UploadUtil,
    HierarchicalListViewModel,

// template
    template,
    dropZoneTemplate,

// Resources
    res
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin, _WidgetCommandBinderMixin], {
        // summary:
        //      Base widget for displaying the panel with tree on top and list for displaying leaf's items, as well as the search box on very top.
        // tags:
        //      internal abstract

        contextMenu: null,

        templateString: template,

        showRoot: false,

        // enableDndFileDropZone: [public] Boolean
        //      Flag to indicate this widget allowed to show drop zone widget ("epi-cms/widget/FilesUploadDropZone" that care about native DnD files from browser) or not.
        enableDndFileDropZone: false,

        // showCreateContentArea: [public] Boolean
        //      Flag to indicate this widget allowed to show create content area by default or not.
        showCreateContentArea: false,

        triggerContextChange: false,

        res: res,

        modelClassName: HierarchicalListViewModel,

        // hierarchicalListClass: [readonly] String
        //      The CSS class to be used on the hierarchical list.
        hierarchicalListClass: "",

        // noDataMessage: [public] String
        //      No data message for list widget
        noDataMessage: "",

        // createContentIcon: [public] String
        //      The icon class to be used in the create content area of the list.
        createContentIcon: "",

        // createContentText: [public] String
        createContentText: "",

        // _dropZoneCreated: boolean
        //      Flag to indicate whether Drop Zone has been created and added, to avoid of create it twice.
        _dropZoneCreated: false,

        modelBindingMap: {
            "searchRoots": ["searchRoots"],
            "searchArea": ["searchArea"],
            "listQuery": ["listQuery"],
            "currentTreeItem": ["currentTreeItem"],
            "currentListItem": ["currentListItem"],
            "editingListItem": ["editingListItem"],
            "showAllLanguages": ["showAllLanguages"]
        },

        _setSearchAreaAttr: function (value) {
            this.searchBox.set("area", value);
        },

        _setSearchRootsAttr: function (value) {
            this.searchBox.set("searchRoots", value);
        },

        _setListQueryAttr: function (value) {
            this.list.set("query", value);
        },

        _setShowAllLanguagesAttr: function (value) {
            this.searchBox.set("filterOnCulture", !value);
        },

        _setCurrentTreeItemAttr: function (value) {
            if (value) {
                this._toggleActive(this.tree, this.list);

                when(this.tree.selectContent(value.toString()), lang.hitch(this, function () {
                    var selectedNode = this.tree.getNodesByItem(value.toString())[0];
                    if (selectedNode) {
                        this.tree.model.set("previousSelection", {
                            selectedContent: selectedNode,
                            selectedAncestors: selectedNode.getTreePath()
                        });
                    }
                }));
            }
        },

        _setCurrentListItemAttr: function (value) {
            this.list.set("currentItem", value);
            this._toggleActive(this.list, this.tree);
        },

        _setEditingListItemAttr: function (value) {
            this.list.set("editingItem", value);
        },

        _setShowRootAttr: function (value) {
            this.tree.set("showRoot", value);
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model && !this.commandProvider && this.modelClassName) {
                var modelClass = declare(this.modelClassName);
                this.model = this.commandProvider = new modelClass({ repositoryKey: this.repositoryKey });
            }
            this.setupContextMenu();
        },

        setupContextMenu: function () {
            // summary: set up the context menu
            //      
            // tags:
            //      public
            this.own(this.contextMenu = new ContextMenu({ category: "context" }));
            this.contextMenu.addProvider(this.model);
        },

        postCreate: function () {
            this.inherited(arguments);

            domClass.add(this.domNode, this.hierarchicalListClass);

            this._setupTree();
            this._setupCreateContentArea();
            this._setupList();

            if (this.enableDndFileDropZone) {
                this._setupDropZone();
            }

            this._toggleCreateContentArea(this.showCreateContentArea);

            this.model.startup();

            // Watching instead of binding since we need the old value as well
            this.own(
                this.model.clipboardManager.watch("data", lang.hitch(this, this._updateClipboardIndicators))
            );
        },

        startup: function () {
            this.inherited(arguments);

            if (this.contextMenu) {
                this.contextMenu.startup();
            }
        },

        destroy: function () {
            this.model.destroy();

            this.inherited(arguments);
        },

        layout: function () {

            // Add class to show drop area
            if (this.enableDndFileDropZone) {
                var grid = this.list.grid;
                if (grid) {
                    domClass.add(grid.bodyNode, "epi-drop-area");
                }
            }

            var listSize = domGeometry.getMarginBox(this.list.domNode),
                searchBoxSize = domGeometry.getMarginBox(this.searchBox.domNode),
                appropriatedHeight = this._contentBox.h - searchBoxSize.h - 32;

            // Re-calculate the height for BlockList to prevent it from taking 100% height
            // "32px" is an appropriate height to show a folder tree node
            if (listSize.h > appropriatedHeight) {
                this.list.resize({ h: appropriatedHeight });
            }

            this.container.resize(this._contentBox);
        },

        _toggleCreateContentArea: function (display) {
            // summary:
            //      Toggle display of the create content area
            // display: [Boolean]
            // tags:
            //      protected

            domStyle.set(this.createContentArea, {
                "display": display ? "" : "none"
            });
        },

        _toggleActive: function (activeWidget, inactiveWidget) {
            // summary:
            //      Toggle visual indication between tree and list
            //
            // activeWidget:
            //      The widget to enable active visuals for
            //
            // inactiveWidget:
            //      The widget to disabe active visuals for

            domClass.remove(inactiveWidget.domNode, "epi-focused");
            domClass.add(activeWidget.domNode, "epi-focused");
        },

        _showNotification: function (message) {
            // summary:
            //      Uses the alert dialog implementation to show notification with the supplied message
            //
            // message:
            //      html encoded string to show in notification dialog
            //
            // tags:
            //      public callback

            var dialog = new Alert({
                destroyOnHide: true,
                description: epistring.toHTML(message)
            });

            dialog.show();
        },

        _onDrop: function (evt, fileList) {
            fileList = UploadUtil.filterFileOnly(fileList);
            if (!fileList || fileList.length <= 0) { // ignore folder dropping
                return;
            }

            var uploadCommand = this.model.getCommand("uploadDefault");
            if (!uploadCommand) {
                return;
            }

            uploadCommand.set("fileList", fileList);
            uploadCommand.execute();
        },

        _setupDropZone: function () {
            // summary:
            //      Setup drop zone for entire widget.
            // tags:
            //      private

            if (!(UploadUtil.supportNativeDndFiles() && !this._dropZoneCreated)) {
                return;
            }

            // Create drop zone if it has not been created)
            var dropZone = new DropZone({ "templateString": dropZoneTemplate, "res": res, "outsideDomNode": this.container.domNode });
            domConstruct.place(dropZone.domNode, this.domNode, "first");

            this.connect(dropZone, "onDrop", this._onDrop);
            this._dropZoneCreated = true;

            var uploadCommand = this.model.getCommand("uploadDefault");
            if (uploadCommand) {
                this.own(
                    uploadCommand.watch("canExecute", function (name, oldValues, newValues) {
                        dropZone.set("enabled", newValues); // disable drop zone in case does not have permission to upload.
                    })
                );
            }
        },

        _setupTree: function () {
            // summary:
            //      Implementation should implement this, to setup the content tree. For example, command binding.
            //
            // tags:
            //      protected

            var registry = this.model._commandRegistry;

            function treeClickHandler(item, node, e) {
                this.model.onTreeItemSelected(item, this.tree.model.isTypeOfRoot(item));
            }

            function copyOrCuthandler(copy) {
                copy ? registry.copy.command.execute() : registry.cut.command.execute();
            }

            this.own(
                on(this.tree, "copyOrCut", copyOrCuthandler),
                on(this.tree, "select", lang.hitch(this, treeClickHandler)),
                on(this.tree, "paste", function () { registry.paste.command.execute(); }),
                on(this.tree, "delete", function () { registry["delete"].command.execute(); }),
                aspect.after(this.tree, "onClick", lang.hitch(this, treeClickHandler), true)
            );

            this.model._commandRegistry.rename.command.set("renameDelegate", lang.hitch(this, function (model) {
                this.tree.getNodesByItem(model)[0].edit();
            }));
        },

        _setupCreateContentArea: function () {
            // summary:
            //      Setup create content area for list widget
            // tags:
            //      protected

            domClass.add(this.createContentAreaIcon, this.createContentIcon);
            this.createContentAreaText.innerHTML = this.createContentText;

            this.list.set("noDataMessage", this.noDataMessage);
        },

        _setupList: function () {
            // summary:
            //      Implementation should implement this, to setup the content list. For example, command binding.
            //
            // tags:
            //      protected
            var registry = this.model._commandRegistry;

            this.own(
                on(this.list, "itemSelected", lang.hitch(this.model, this.model.onListItemSelected)),
                aspect.after(this.model, "onListItemUpdated", lang.hitch(this.list, function (deferred) {
                    when(deferred, lang.hitch(this, this.editContent));
                }))
            );

            // TODO: When it possible to do a breaking change we should remove the changeContextOnItemSelection variable and
            //       create a callback method that can be overriden in child classes, instead of calling the command explicitly.
            if (this.model.changeContextOnItemSelection) {
                this.own(on(this.list, "itemAction", function() {
                    registry.edit.command.execute();
                }));
            }
        },

        _onSearchAction: function (value) {
            // summary:
            //      Implementation should implement this, to process search action.
            //
            // value:
            //      Search string
            //
            // tags:
            //      protected

            this.model.onSearch(value.metadata);
        },

        // =======================================================================
        // Clipboard handling

        _updateClipboardIndicators: function (name, oldValues, newValues) {

            var isCopy = this.model.clipboardManager.isCopy();

            if (oldValues instanceof Array) {
                array.forEach(oldValues, function (clip) {
                    this.tree.toggleCut(clip.data, false);
                    this.list.toggleCut(clip.data, false);
                }, this);
            }

            if (!isCopy && newValues instanceof Array) {
                array.forEach(newValues, function (clip) {
                    this.tree.toggleCut(clip.data, true);
                    this.list.toggleCut(clip.data, true);
                }, this);
            }
        }

    });

});
