﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Stateful",
    "dojo/when",
    "dojox/html/entities",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/CheckedMenuItem",
    "epi/dependency",
    "epi/shell/_StatefulGetterSetterMixin",
    // Resouces
    "dojo/text!./templates/PersonalizationSelector.html",
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.editors.contentarea.personalize",
    // Widgets used in template
    "dijit/Menu",
    "epi/shell/widget/RadioMenuItem"
], function(
    array,
    declare,
    lang,
    Stateful,
    when,
    entities,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    CheckedMenuItem,
    dependency,
    _StatefulGetterSetterMixin,
    // Resouces
    template,
    resources
) {

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
        // tags:
        //      internal

        templateString: template,

        model: null,

        resources: resources,

        postMixInProperties: function() {
            this.inherited(arguments);

            if(!this.store) {
                var registry = dependency.resolve("epi.storeregistry");
                this.store = registry.get("epi.cms.visitorgroup");
            }
        },

        postCreate: function() {
            this.inherited(arguments);

            this.own(
                this.rdEveryone.on("change", lang.hitch(this, "restoreDefault")),
                this.model.watch("roleIdentities", lang.hitch(this, function () {
                    if (!this.rdEveryone.checked && this.model.roleIdentities && this.model.roleIdentities.length === 0) {
                        this.restoreDefault();
                    }
                }))
            );

            //Query the store for all groups
            when(this.store.query(), lang.hitch(this, function(groups){
                if(!this._destroyed) {
                    this.set("visitorGroups", groups);
                }
            }));
        },

        restoreDefault: function() {
            array.forEach(this.visitorGroupsMenu.getChildren(), function (item) {
                item.set("checked", false);
            });

            this.rdEveryone.set("checked", true);
            this.model.set("roleIdentities", null);
        },

        _setVisitorGroupsAttr: function(groups) {
            this._set("visitorGroups", groups);

            if(!this.model.hasAnyRoles()) {
                this.rdEveryone.set("checked", true);
            }

            array.forEach(groups, function(group) {

                var item = new CheckedMenuItem({
                    label: entities.encode(group.name),
                    visitorgroupId: group.id,
                    checked: this.model.isRoleSelected(group.id)
                });

                this.own(item.watch("checked", lang.hitch(this, function(property, oldValue, newValue){
                    this.model.selectRole(group.id, newValue);

                    this.rdEveryone.set("checked", !this.model.hasAnyRoles());
                })));

                this.visitorGroupsMenu.addChild(item);
            }, this);
        }
    });
});