﻿define([
// dojo
    "dojo",
    "dojo/_base/lang",
    "dojo/_base/array",
// epi
    "epi",
    "epi/dependency",

    "epi-cms/ApplicationSettings",
    "epi-cms/widget/_ItemSelectorBase",
    "epi-cms/widget/_DropDownWidgetItemMixin",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.resolutionselector"],
function (
// dojo
    dojo,
    lang,
    array,
// epi
    epi,
    dependency,

    ApplicationSettings,
    _ItemSelectorBase,
    _DropDownWidgetItemMixin,
//template
    res) {
    return dojo.declare([_ItemSelectorBase, _DropDownWidgetItemMixin], {
        // tags:
        //      internal

        // _resolutionHash: [private] Object
        //    Map the URL resolution name (ex. 1024x768) with the
        //    resolution name (ex. iPad Horizontal 1024x768)
        _resolutionHash: {},

        // defaultItem: [public] Object
        //    The widget's default item to be added to the menu.
        defaultItem: {
            name: res.defaultitem,
            width: 1,
            height: 1
        },

        postMixInProperties: function () {
            // summary:
            //    Sets up the data store.
            //
            // tags:
            //    protected
            this.inherited(arguments);

            // To get correctly the displayResolutions for other ApplicationSetting. 
            this._availableResolutions = lang.clone(ApplicationSettings.displayResolutions);

            this._availableResolutions.unshift(this.defaultItem);

            array.forEach(this._availableResolutions, lang.hitch(this, function (resolution) {
                var urlName = resolution.width + "x" + resolution.height;
                this._resolutionHash[urlName] = resolution.name;
            }));

            this.checkedItem  = (this.checkedItem ? this._resolutionHash[this.checkedItem] : this.defaultItem.name);
        },

        postCreate: function () {
            // summary:
            //    Raised when after the widget creating, it calls
            //    the setup function.
            //
            // tags:
            //    protected
            this.inherited(arguments);
            this.set("header", res.header);
            this.set("items", this._availableResolutions);
        },

        getDefaultResolution: function (/*Object*/channel) {
            // summary:
            //    Get the default resolution given a channel.
            //
            // channel:
            //    DisplayChannel to get the resolution.
            //
            // tags:
            //    private
            var defaultResolution = this.defaultItem;

            var resolutions = dojo.filter(this._availableResolutions, function (resolution) {
                if (resolution.id == channel.resolutionId) {
                    return resolution;
                }
            });
            if (resolutions != null && lang.isArray(resolutions) && resolutions.length >= 1) {
                defaultResolution = resolutions[0];
            }
            return defaultResolution;
        }

    });
});