﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/Deferred",
    "dojo/when",
// epi
    "epi/dependency",
    "epi-cms/widget/_ItemSelectorBase",
    "epi-cms/widget/_DropDownWidgetItemMixin",
    "epi-cms/component/SiteTreeModel",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.viewlanguageselector"
], function (
    declare,
    lang,
    array,
    Deferred,
    when,
// epi
    dependency,
    _ItemSelectorBase,
    _DropDownWidgetItemMixin,
    SiteTreeModel,
// resources
    resources
) {
    return declare([_ItemSelectorBase, _DropDownWidgetItemMixin], {
        // tags:
        //      internal

        // _viewLanguageHash: [private] Object
        //    Map the int id with its name
        _viewLanguageHash: {},

        // _isFinished: [private] dojo.Deferred
        //    Deferred to indicate when widget finished its setup process.
        _isFinished: null,

        // _siteTreeModel: Object
        //      Instance of SiteTreeModel
        _siteTreeModel: null,

        postCreate: function () {
            // summary:
            //    Raised after the widget creating, sets the header and
            //    load the items to be added to the menu.
            //
            // tags:
            //    protected

            this.inherited(arguments);
            this.set("header", resources.header);

            if (!this._siteTreeModel) {
                var registry = dependency.resolve("epi.storeregistry"),
                    siteStructureStore = registry.get("epi.cms.sitestructure");
                this._siteTreeModel = new SiteTreeModel({store: siteStructureStore});
            }
        },

        fetchData: function () {
            // summary:
            //      Load all available languages of current site
            // tags:
            //      protected

            var deferred = new Deferred();

            when(this._siteTreeModel.getCurrentSite(), lang.hitch(this, function (site) {
                if (site) {
                    when(this._siteTreeModel.getChildren({ url: site.currentSite }, lang.hitch(this, function (languages) {
                        this._createHashMap(languages);

                        // manually update defaultItem name
                        if (this.defaultItem) {
                            this.defaultItem.name = this._viewLanguageHash[this.defaultItem.languageId] || this.defaultItem.name;
                        }

                        this.set("items", languages);

                        deferred.resolve({ status: true });
                    })));
                }
            }));

            return deferred.promise;
        },

        checkItem: function (/*Object*/item) {
            // summary:
            //    Mark a specific item as checked.
            //
            // item:
            //    The item to be selected. If not specified
            //    all the radio buttons will be unchecked.
            //
            // tags:
            //    public

            if (!item || !this._viewLanguageHash[item.languageId]) {
                item = this.get("currentSelectedItem");
            }

            this.inherited(arguments);
        },

        _getCurrentSelectedItemAttr: function () {
            // summary:
            //      Custom get currentSelectedItem method, loop through all candidates, check exist or not
            // tags:
            //      Public

            var foundCandidate = null;
            array.some([this.currentSelectedItem, this.checkedItem, this.defaultItem], lang.hitch(this, function (candidate) {
                if (candidate && (this._viewLanguageHash[candidate.languageId || candidate])) {
                    foundCandidate = candidate.languageId ? candidate : { languageId: candidate, name: this._viewLanguageHash[candidate] };
                    return true;
                }

                return false;
            }));

            return foundCandidate;
        },

        _createHashMap: function (items) {
            // summary:
            //      Creates a map with the language id and its name
            // tags:
            //      Private

            // Clear hash
            this._viewLanguageHash = {};

            // Create has mapping
            array.forEach(items, lang.hitch(this, function (item) {
                this._viewLanguageHash[item.languageId] = item.name;
            }));
        }
    });
});