﻿define([
// dojo
    "dojo",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/connect",
    "dojo/_base/Deferred",
    "dojo/dom-class",
// dijit
    "dijit/DropDownMenu",
    "dijit/form/DropDownButton",
    "dijit/focus",
// epi
    "epi",
    "epi/dependency",
    "epi-cms/widget/VisitorGroupsSelector",
// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.visitorgroupsbutton"],
function (
// dojo
    dojo,
    declare,
    lang,
    connect,
    Deferred,
    domClass,
// dijit
    DropDownMenu,
    DropDownButton,
    focusUtil,
// epi
    epi,
    dependency,
    VisitorGroupsSelector,
    resources
) {
    return declare([DropDownButton], {
        // tags:
        //      internal

        // _contextMenu: [private] dijit.DropDownMenu
        //    The context menu for the button.
        _contextMenu: null,

        // baseClass: [public] String
        //    CSS base classes for the button.
        baseClass: "epi-mediumButton epi-chromelessButton",

        // iconClass: [public] String
        //    CSS class that adds the button icon.
        iconClass: "epi-iconUsers",

        // contextMenuProperties [public] Object
        //    Context menu properties.
        contextMenuProperties: {},

        // _visitorGroupsSelector: [private] Object
        //    The visitor groups selector widget.
        _visitorGroupsSelector: null,

        // _viewSettingsManager: [private] Object
        //    The view manager
        _viewSettingsManager: null,

        postMixInProperties: function () {
            // summary:
            //    Mixin the widget properties.
            //
            // tags:
            //    protected
            this.inherited(arguments);

            lang.mixin(this.contextMenuProperties, {
                "class": "epi-DropDownMenu"
            });
        },

        postCreate: function () {
            // summary:
            //    Raised after the widget is created. It will create
            //    the widget's context menu and add its children.
            //
            // tags:
            //    protected
            this.inherited(arguments);

            this._viewSettingsManager = dependency.resolve("epi.viewsettingsmanager");
            var viewSettingVisitorGroup = this._viewSettingsManager.getSettingProperty("visitorgroup");

            if (!this.dropDown) {

                this.dropDown = dijit.DropDownMenu(this.contextMenuProperties);

                this._visitorGroupsSelector = new VisitorGroupsSelector({ checkedItem: viewSettingVisitorGroup });
                connect.connect(this._visitorGroupsSelector, "onItemSelected", this, this._onVisitorGroupSelected);

                this._addChildren(this._visitorGroupsSelector);

                Deferred.when(this._visitorGroupsSelector.isSetupFinished(), lang.hitch(this, function (result) {
                    this._activeButton(!this._isDefaultSelection());
                    this._updateTooltip();
                }));
            }

            connect.connect(this.dropDown, "onOpen", this, function (e) {
                focusUtil.focus(this._visitorGroupsSelector.menuNode.domNode);
            });
        },

        _updateTooltip: function () {

            var tooltiptext = null;

            if (this._isDefaultSelection()) {
              tooltiptext = resources.defaulttooltip;
            }
            else {
                var selectedItem = this._visitorGroupsSelector.currentSelectedItem;
                tooltiptext = lang.replace(resources.tooltip, { visitorgroup: selectedItem.name });
            }
            this.set("title", tooltiptext);
        },

        _isDefaultSelection: function () {
            // summary:
            //    Checks if the selected items on both widgets are the default items.
            //
            // tags:
            //    private
            return this._visitorGroupsSelector.isDefaultItemSelected();
        },

        _onVisitorGroupSelected: function (/*Object*/visitorGroup) {
            // summary:
            //    Raised when a visitor group is selected.
            //
            // visitorGroup:
            //   The selected visitor group
            //
            // tags:
            //    private event
            this._changeView(visitorGroup);
        },

        _changeView: function (/*Object*/visitorGroup) {
            // summary:
            //    Change the view to the given resolution.
            //
            // visitorGroup:
            //    Visitor group object
            //
            // tags:
            //    private
            if (visitorGroup) {
                if (visitorGroup.name == this._visitorGroupsSelector.defaultItem.name) {
                    connect.publish("/epi/cms/action/viewsettingvaluechanged", ["visitorgroup", null]);
                } else {
                    connect.publish("/epi/cms/action/viewsettingvaluechanged", ["visitorgroup", visitorGroup.id]);
                }
            }
            this._activeButton(!this._isDefaultSelection());
            this._updateTooltip();
        },

        _addChildren: function (/*Object*/item) {
            // summary:
            //    Populate the context menu with the configured items.
            //
            // item:
            //    The item that will be added to the dropdown
            //
            // tags:
            //    private
            this.dropDown.addChild(item);
        },

        _activeButton: function (/*Boolean*/active) {
            // summary:
            //      Set active state for button widget
            //
            // active:
            //    States if the button will be checked or not.
            //
            // tags:
            //      private
            if (!this.dropDown) { return; }
            if (active) {
                domClass.add(this.domNode, "dijitChecked");
            } else {
                domClass.remove(this.domNode, "dijitChecked");
            }
        }
    });

});