﻿define([
// dojo
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
// dojox
    "dojox/html/entities",
// dijit
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/Menu",
// epi
    "epi",
    "epi/dependency",
    "epi/shell/widget/RadioMenuItem",
// template
    "dojo/text!./templates/_ItemSelector.html"],
function (
// dojo
    connect,
    declare,
    lang,
    array,
// dojox
    htmlEntities,
// dijit
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Menu,
// epi
    epi,
    dependency,
    RadioMenuItem,
// template
    template) {
    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
        // tags:
        //      internal

        // templateString: [protected] String
        //    The widget template.
        templateString: template,

        // templateString: [protected] String
        //    The widget template.
        baseClass: "epi-invertedTooltip",

        // _store: [private] Object
        //    The widget data store.
        _store: null,

        // currentSelectedItem: [public] Object
        //    The current selected item.
        currentSelectedItem: null,

        // checkedItem: [public] String
        //    The name of the item that should be checked.
        checkedItem: null,

        // defaultItem: [protected] String
        //    The default item on the menu.
        defaultItem: null,

        // _items: [private] Array
        //    Items to be added on the menu.
        _items: [],

        isDefaultItemSelected: function () {
            // summary:
            //    Verify is the current selected item is the default item.
            //
            // tags:
            //    public
            return (this.currentSelectedItem && this.defaultItem && this.currentSelectedItem.name == this.defaultItem.name);
        },

        postCreate: function () {
            this.inherited(arguments);
            this.own(connect.connect(this.menuNode, "onItemHover", this, function (item) {
                this.menuNode.focusChild(item);
            }));
        },

        checkItem: function (/*Object*/item) {
            // summary:
            //    Mark a specific item as checked.
            //
            // item:
            //    The item to be selected. If not specified
            //    all the radio buttons will be unchecked.
            //
            // tags:
            //    public
            array.forEach(this.menuNode.getChildren(), lang.hitch(this, function (child) {
                child.set('checked', false);
                var itemvalue = child.value;
                if (item != null && itemvalue.name == item.name) {
                    child.set('checked', true);
                    this.currentSelectedItem = child.value;
                }
            }));
        },

        _setupUI: function () {
            // summary:
            //    Setup the UI loading the items on the menu.
            //
            // tags:
            //    private

            // Remove all child before addnew
            array.forEach(this.menuNode.getChildren(), lang.hitch(this, function (child) {
                this.menuNode.removeChild(child);
            }));

            var items = this.get("items");
            array.forEach(items, lang.hitch(this, function (item) {
                this.menuNode.addChild(this._createMenuItem(item));
            }));
        },

        _onItemSelected: function (item) {
            // summary:
            //   Raised when a item is selected, it will fire the view setting change.
            //
            // tags:
            //    event
            this.currentSelectedItem = item.value;
            this.onItemSelected(item.value);
        },

        onItemSelected: function (item) {
            // summary:
            //   Raised when a item is selected, it will fire the view setting change.
            //
            // tags:
            //    event
        },

        _createMenuItem: function (/*Object*/item) {
            // summary:
            //    Create a RadioMenuItem
            //
            // item:
            //    The item to create the radio button from.
            //
            // tags:
            //    private
            var menuItem = new RadioMenuItem({
                checked: (item.name == this.checkedItem),
                label: htmlEntities.encode(item.name),
                value: item,
                onClick: lang.hitch(this, function () {
                    this._onItemSelected(menuItem);
                })
            });

            if (menuItem.checked) {
                this.currentSelectedItem = menuItem.value;
            }

            return menuItem;
        },

        _setItemsAttr: function (/*Array*/value) {
            // summary:
            //    Sets the _item property
            //
            // value:
            //    The value to be set to the property.
            this._set("_items", value);
            this._setupUI();
        },

        _getItemsAttr: function () {
            // summary:
            //    Gets the _item property
            return this.get("_items");
        },

        // summary:
        //    Setter for the widget's header.
        _setHeaderAttr: { node: "header", type: "innerHTML" }

    });

});
