﻿define([
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/promise/all",
    "dojo/Stateful",

// epi 
    "epi/dependency",
    "epi/shell/_StatefulGetterSetterMixin",
    "epi-cms/contentediting/ContentActionSupport",
    "epi/shell/_ContextMixin",
    "epi-cms/ContentLanguageHelper",

// resources
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.versionstatus"
],
function (
// dojo
    array,
    declare,
    Deferred,
    lang,
    when,
    all,
    Stateful,

// epi
    dependency,
    _StatefulGetterSetterMixin,
    ContentActionSupport,
    _ContextMixin,
    ContentLanguageHelper, 

// resources
    versionStatusResources
) {

    return declare([Stateful, _ContextMixin], {
        // tags:
        //      internal

        _contentStructureStore: null,

        // autoLoadStatus: [Boolean]
        //      Set it to true of the status information should be automatically loaded when the contentLink is set
        // tags:
        //      public
        autoLoadStatus: false,

        // statusIcon: [String]
        //      Store content status icon classes
        // tags:
        //      public
        statusIcon: null,

        // statusMessage: [Object]
        //      Store content status message.
        //      Can be [String] or [Array] data type.
        // tags:
        //      public
        statusMessage: null,

        constructor: function() {
             this._statusIconArray = [
            // Not created icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator0"],
            // Rejected icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator1"],
            // Checked-out icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator2"],
            // Checked-in icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator3"],
            // Published icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator4"],
            // Previously published icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator5"],
            // Delayed publish icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator6"],
            // Expired icon CSS classes
                    ["epi-statusIndicatorIcon", "epi-statusIndicator7"]
                ];

            this._statusMessageArray = [
            // Not created message
                    versionStatusResources.notcreated,
            // Rejected message
                    versionStatusResources.rejected,
            // Checked-out message
                    versionStatusResources.checkedout,
            // Checked-in message
                    versionStatusResources.checkedin,
            // Published message
                    versionStatusResources.published,
            // Previously published message
                    versionStatusResources.previouslypublished,
            // Delayed publish message
                    versionStatusResources.delayedpublish,
            // Expired message
                    versionStatusResources.expired
                ];
        },

        postscript: function () {
            this.inherited(arguments);

            this._contentStructureStore = this._contentStructureStore || dependency.resolve("epi.storeregistry").get("epi.cms.content.light");
        },

        loadStatus: function() {
            // summary:
            //      Loads the status for the current contentLink
            // tags:
            //      public
            if (!this.contentLink) {
                return;
            }
            when(this.getCurrentContext(), lang.hitch(this, function(ctx) {
                //- Using refresh instead get function to load content link 
                //- Do not want use cache in this case
                when(this._contentStructureStore.refresh(this.contentLink), lang.hitch(this, function(contentInfo) {
                    if (contentInfo && contentInfo.status !== undefined) {
                        this.set("content", contentInfo);
                    }
                }));
            }));
        },

        _contentLinkSetter: function (/* String */contentLink) {
            // summary:
            //      Content Link setter
            // contentLink: [String]

            if (!contentLink) {
                return;
            }

            this.contentLink = contentLink;

            if(this.autoLoadStatus){
                this.loadStatus();
            }
        },

        _isDeletedGetter: function() {
            return this.get("content") && this.get("content").isDeleted;
        },

        _isTranslationNeededGetter: function() {
            return this.get("content") && this.get("content").missingLanguageBranch && this.get("content").missingLanguageBranch.isTranslationNeeded;
        },

        _missingLanguageBranchGetter: function() {
            return this.get("content") && this.get("content").missingLanguageBranch;
        },

        _contentSetter: function (content) {
            this.content = content;
            
            this.set("contentStatus", content.status);
        },

        _contentStatusSetter: function(status) {
            this.contentStatus = status;

            if(!status) {
                return;
            }

            this.set("statusIcon", this._statusIconArray[status]);

            when(this._getStatusMessage(), lang.hitch(this, function(message) {
                this.set("statusMessage", message);
            }));
        },

        _getStatusMessage: function () {
            // summary:
            //      Get content version status message.
            //      Override this function to customize content status message.
            // tags:
            //      protected

            //If the content is deleted return version not found
            if (this.get("isDeleted")) {
                return versionStatusResources.versionnotfound;
            }
            if (this.get("isTranslationNeeded")) {
                return ContentLanguageHelper.getMissingLanguageMessage(this.content);
            }

            if(this.get("isVisibleOnSite")){
                return null;
            }
            return this._statusMessageArray[this.get("contentStatus")];
        },

        _isVisibleOnSiteGetter: function () {
            // summary:
            //      Allow create additional information area or not.
            //      Based on content version status.
            // status: [Integer]
            //      Content status
            // tags:
            //      public

            var missingLanguageBranch = this.get("missingLanguageBranch");
            if (this.get("isDeleted") || missingLanguageBranch && missingLanguageBranch.reason == 6) {
                return false;
            }

            return this.get("contentStatus") === ContentActionSupport.versionStatus.Published;
        }
    });
});
