﻿define([
    "dojo/_base/declare",
    "epi/shell/command/_Command"
], function (declare, _Command) {

    return declare([_Command], {
        // summary:
        //      A base class implementation for commands which toggles the value of a property on the model.
        //
        // tags:
        //      public

        // property: [public] String
        //		The name of the property on the model which this command will toggle.
        property: null,

        // active: [readonly] Boolean
        //		Flag which indicates whether this command is active. Value is true if command is active; otherwise false.
        active: false,

        _execute: function () {
            // summary:
            //		Toggles the value of the given property on the model.
            // tags:
            //		protected

            var name = this.property,
                value = !this.active;

            this.model.set(name, value);
            this.set("active", value);
        },

        _onModelChange: function () {
            // summary:
            //		Updates canExecute after the model has been updated.
            // tags:
            //		protected

            var model = this.model,
                active = model && model[this.property],
                canExecute = typeof active == "boolean";

            this.set("canExecute", canExecute);
            this.set("active", canExecute && active);
        }
    });
});