﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dijit/Menu",
    "dijit/form/DropDownButton",
    "epi/shell/command/builder/MenuBuilder"
], function (array, declare, lang, Menu, DropDownButton, MenuBuilder) {

    return declare([MenuBuilder], {
        // summary:
        //      Builds a drop down button widget with commands as child menu items.
        //
        // tags: private

        _addToContainer: function (widget, container) {
            // summary:
            //		Adds a drop down button to the container if not added already, and
            //		adds the widget to the drop down button.
            // tags:
            //		protected

            if (!this._added) {
                this._button = new DropDownButton(lang.mixin({ dropDown: new Menu() }, this.settings));
                this._button._commandCategory = widget._commandCategory;

                if (container.addChild) {
                    container.addChild(this._button);
                } else {
                    this._button.placeAt(container);
                    this._button.startup();
                }

                this._added = true;
            }

            return this.inherited(arguments, [widget, this._button.dropDown]);
        },

        remove: function (command, container) {
            // summary:
            //      Remove the ui representation of a specific command from a container
            // command: epi/shell/command/_Command
            //      The command to remove
            // container: dijit/_Widget
            //      The container displaying the command
            // tags:
            //      public

            var button = this._button,
                children = button.dropDown ? button.dropDown.getChildren() : [];

            array.forEach(children, function (child) {
                if (child._command === command) {
                    button.removeChild(child);
                    child.destroy();
                    return true;
                }
            });
            return false;
        }
    });
});
