﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-class",

    "epi/shell/command/_CommandModelBindingMixin",
    "epi/shell/command/builder/_Builder",

    "epi/shell/widget/ListContainer",
    "epi/shell/widget/ExpandoButton"
],

function (
    declare,
    lang,

    domClass,

    _CommandModelBindingMixin,
    _Builder,

    ListContainer,
    ExpandoButton
) {

    return declare([_Builder], {
        // tags: private

        _create: function (/*Object*/command) {
            // tags:
            //      protected, override

            // Intermediate storage in d is intended to shake the doc parser 
            // off since it cannot resolve dynamic dependencies.
            var d = { declare: declare }; 
            var widgetPrototype = d["declare"]([command.settings.widget, _CommandModelBindingMixin]),
                widget = new widgetPrototype(lang.mixin(command.settings, this.itemSettings));

            return widget;
        },

        _addToContainer: function (/*Object*/widget, /*Object*/container) {
            // tags:
            //      protected, override

            this._createListContainer(widget, container);

            if (this._containerAdded) {
                if (widget.model) {
                    // Watch available of each child item in list in order to hide list container when its all children not visible
                    widget.model.watch("isAvailable", lang.hitch(this, function () {
                        if (this._listContainer.updateVisibility) {
                            this._listContainer.updateVisibility();
                        }
                    }));
                }
            }

            this.inherited(arguments, [widget, this._listContainer]);

            if (this._listContainer.getChildren().length > 1) {
                domClass.add(widget.getParent().domNode, "epi-leftSeparator");
            }
        },

        _createListContainer: function (/*Object*/widget, /*Object*/container) {
            // tags:
            //      private

            if (this._containerAdded) {
                return;
            }

            this._listContainer = new ListContainer(this.settings);
            this._listContainer._commandCategory = widget._commandCategory;

            if (container.addChild) {
                container.addChild(this._listContainer);
            } else {
                this._listContainer.placeAt(container);
                this._listContainer.startup();
            }

            this._containerAdded = true;
        }
    });
});