﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dijit/Menu",
    "dijit/MenuItem",
    "dijit/CheckedMenuItem",
    "dijit/PopupMenuItem",
    "epi/shell/command/builder/_Builder",
    "epi/shell/command/_CommandModelBindingMixin"
], function (array, declare, lang, on, Menu, MenuItem, CheckedMenuItem, PopupMenuItem, _Builder, _CommandModelBindingMixin) {

    return declare([_Builder], {
        // summary:
        //      Builds a drop down button widget with commands as child menu items.
        // tags:
        //      internal

        // _menuItemClass: [private] class
        //      The class used for representing a menu item
        _menuItemClass: declare([MenuItem, _CommandModelBindingMixin]),

        // _checkedMenuItemClass: [private] class
        //      The class used for representing a checked menu item for a checked command
        _checkedMenuItemClass: declare([CheckedMenuItem, _CommandModelBindingMixin]),

        // _popupMenuItemClass: [private] class
        //      The class used for representing a popup menu item for a option command
        _popupMenuItemClass: declare([PopupMenuItem, _CommandModelBindingMixin]),

        _create: function (command) {
            // summary:
            //		Builds a menu item widget from the given command.
            // tags:
            //		public

            var isToggle = typeof command.active == "boolean",
                isOption = lang.isArray(command.options),
                isPopup = "popup" in command,
                cls = isToggle ? this._checkedMenuItemClass : (isOption || isPopup) ? this._popupMenuItemClass : this._menuItemClass,
                options = lang.mixin({ model: command }, this.itemSettings);

            var widget = new cls(options);

            // If creating a command that already has a popup menu then associate it with the widget.
            if (isPopup) {
                if (command.popupClass) {
                    // Construct the popup class defined in the command.
                    command.set("popup", new command.popupClass({ category: command.popupCategory }));
                }
                widget.popup = command.popup;
            }

            // If creating a command that has options then create the options on the widget.
            if (isOption) {
                this._createOptionMenu(command, widget);
            }

            return widget;
        },

        _createOptionMenu: function (command, widget) {
            // summary:
            //		Helper method to add options from an OptionCommand to a popup menu.
            // tags:
            //		private

            widget.popup = new Menu();
            array.forEach(command.options, function (option) {
                var child = new CheckedMenuItem({
                    label: option.label,
                    checked: command.selected === option.value
                });
                child.own(on(child, "click", function () {
                    command.set("selected", option.value);
                }));
                child.own(command.watch("selected", function (name, oldValue, newValue) {
                    child.set("checked", newValue === option.value);
                }));
                widget.popup.addChild(child);
            });
        }
    });
});
