﻿define([
"dojo/_base/array",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/dom-class",
"dojo/dom-construct",
"dojo/dom-style",
"dijit/_WidgetBase",
"dijit/_Container",
"dijit/_Contained",
"dijit/_TemplatedMixin",
"dijit/layout/_LayoutWidget"
],
function (
    array,
    declare,
    lang,
    domClass,
    domConstruct,
    domStyle,
    _WidgetBase,
    _Container,
    _Contained,
    _TemplatedMixin,
    _LayoutWidget
) {
    var parentItem = declare([_WidgetBase, _Container, _Contained, _TemplatedMixin], {
            templateString: "<li class='epi-parentContainer'><div class='epi-formsWidgetWrapper' data-dojo-attach-point='containerNode'></div></li>"
        }),
        groupItem = declare([_WidgetBase, _Container, _Contained, _TemplatedMixin], {
            templateString: "<li><div class='epi-formsWidgetWrapper' data-dojo-attach-point='containerNode'></div></li>"
        }),
        fieldItem = declare([_WidgetBase, _Container, _Contained, _TemplatedMixin], {

            labelTarget: "",
            label: "",
            readonlyIconDisplay: null,
            hasFullWidthValue: false,

            templateString: "<li class='epi-formsRow' data-dojo-attach-point='containerNodeWrapper'> \
                                <label data-dojo-attach-point='labelNode'> \
                                    <span class='dijitInline dijitReset dijitIcon epi-iconPenDisabled' data-dojo-attach-point='readonlyIcon'></span> \
                                </label> \
                                <div class='epi-formsWidgetWrapper' data-dojo-attach-point='containerNode'></div> \
                            </li>",

            _setReadonlyIconDisplayAttr: function(/* Boolean */value) {
                domStyle.set(this.readonlyIcon, "display", value === true ? "" : "none");
            },

            _setLabelAttr: function(value) {
                this.labelNode.appendChild(domConstruct.toDom(value));
            }, 
            _setLabelTargetAttr: { node: "labelNode", attribute: "for", type: "attribute" },
            _setTooltipAttr: { node: "labelNode", attribute: "title", type: "attribute" },

            _setHasFullWidthValueAttr: function(/* Boolean */ value) {
                value && domClass.add(this.containerNodeWrapper, "epi-formsRow--fullWidthValue");
            }
        }),
        hiddenFieldItem = declare([_WidgetBase, _Container, _Contained, _TemplatedMixin], {
            templateString: "<li class='epi-formsRow'><div class='epi-formsWidgetWrapper' data-dojo-attach-point='containerNode'></div></li>"
        });

    return declare([_LayoutWidget, _TemplatedMixin], {
        // summary:
        //     Widget that contains form items widgets, aimed to apply the html formatting
        //
        // tags:
        //      internal xproduct

        templateString: '<div class="epi-containerLayout clearfix"><ul data-dojo-attach-point="containerNode"></ul></div>',

        _FieldItem: fieldItem,
   
        _HiddenFieldItem: hiddenFieldItem,
        
        _ParentItem: parentItem,
        
        _GroupItem: groupItem,

        addChild: function (child) {
            var wrapper;
            switch (child._type) {
                case "field":
                    wrapper = new this._FieldItem({
                        labelTarget: child.checkbox ? child.checkbox.id : child.id,
                        label: child.label,
                        tooltip: child.tooltip,
                        readonlyIconDisplay: child.readOnly,
                        hasFullWidthValue: child.useFullWidth
                    });

                    wrapper.own(child.watch("readOnly", function (name, oldValue, newValue) {
                        wrapper.set("readonlyIconDisplay", newValue);
                    }));

                    break;
                    
                case "hiddenfield":
                    wrapper = new this._HiddenFieldItem();
                    
                    break;

                case "parent":
                    wrapper = new this._ParentItem();
                    
                    break;
                    
                case "group":
                    wrapper = new this._GroupItem();
                    
                    break;
            }

            this._addChildToWrapper(wrapper, child);

            this.own(wrapper);

            return this.inherited(arguments, [wrapper]);
        },

        _addChildToWrapper: function (wrapper, child) {
            // summary:
            //      Add a child to wrapper. Overwrite to customize how a child is put into the wrapper.
            // wrapper: dijit/_WidgetBase
            //      The wrappper
            // child: dijit/_WidgetBase
            //      The child

            wrapper.addChild(child);
        },

        layout: function () {
            // Let all the children take whatever size they want;

            array.forEach(this.getChildren(), function (wrapper) {
                var child = wrapper.getChildren()[0];
                if (child && child.resize) {
                    child.resize();
                }
            });
        }
    });

});