﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-class",
    "dojo/dom-geometry",
    "dojo/dom-style",

    "dojo/keys",

    "dojo/text!./templates/ExpandoButton.html",

    "dijit/_KeyNavContainer",
    "dijit/_Widget",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_TemplatedMixin",

// Widgets used in the template
    "dijit/form/ToggleButton",
    "epi/shell/widget/ListContainer"
],

function (
    array,
    declare,
    lang,

    domClass,
    domGeometry,
    domStyle,

    keys,

    template,

    _KeyNavContainer,
    _Widget,
    _WidgetsInTemplateMixin,
    _TemplatedMixin
) {

    return declare([_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _KeyNavContainer], {
        // summary:
        //    An expando button, which contains buttons inside, and will be slide-to-visible when clicking on it.
        //
        // tags:
        //    public

        // templateString: [protected] String
        //      Widget's template string.
        templateString: template,

        // value: [public] Array
        //      List of buttons that this expando button contains.
        buttons: null,

        // Default of toggle eye menu
        isExpand: false,

        // value: [public] String
        //      Represents the label/tooltip of this button.
        label: "",

        iconClass: "",

        hasToggleableChildren: true,

        // _separatorBorderWidth: [private] Integer
        //      Hold the separator border width between the toggle button and its contents
        _separatorBorderWidth: 0,

        postMixInProperties: function () {
            if (!this.showInGroup) {
                this.templateString = '<ul class="dijitReset dijitInline epi-mediumButton epi-expandoButton"><li data-dojo-attach-point="itemsContainer">' + this.templateString + '</li></ul>';
            }

            this.inherited(arguments);
        },

        postCreate: function () {
            // summary:
            //      Post widget creation.
            // tags:
            //      protected

            this.inherited(arguments);

            // register arrow keys for navigation
            this.connectKeyNavHandlers([keys.LEFT_ARROW], [keys.RIGHT_ARROW]);

            // add buttons to buttonContainer
            if (this.buttons) {
                array.forEach(this.buttons, function (button) {
                    this.containerNode.addChild(button);

                    // disable tab navigation
                    this._startupChild(button.getParent());
                    this._startupChild(button);

                    if (!button.get("iconClass")) {
                        domClass.add(button.getParent().domNode, "epi-expandoButtonTextNode");
                    }
                }, this);
            }

            this.subscribe("/epi/shell/widget/ExpandoButton/action/toggle", lang.hitch(this, function (active) {
                this.toggleButtonState.set("checked", active);
            }));

            if (this.iconClass) {
                this.toggleButtonState.set("iconClass", this.iconClass);
            }

            this.connect(this.toggleButtonState, "onClick", this.onClick);

        },

        onClick: function () { },

        startup: function () {
            this.inherited(arguments);

            // We need to get border width before expand/collapse or we will get incorrect value in FireFox
            // because domGeom.position(this.containerNode.domNode).w returns a random width that depends on the button animation
            if (this.showInGroup) {
                this._separatorBorderWidth = domGeometry.getBorderExtents(this.containerNode.domNode).w;
            }

            this.toggleButtonState.set("checked", this.isExpand);
        },

        _getNextFocusableChild: function (/*Object*/child, /*Integer*/dir) {
            // summary:
            //      Returns the next or previous focusable child, compared to "child"
            //      Override this function to fix menu order.
            // child: Widget
            //      The current widget
            // dir: Integer
            //      * 1 = after
            //      * -1 = before

            if (child) {
                child = this._getSiblingOfChild(child, dir);
            }

            var children = [this.togglebuttonState];
            lang.mixin(children, this.containerNode.getChildren());

            for (var i = 0; i < children.length; i++) {
                if (!child) {
                    child = children[(dir > 0) ? 0 : (children.length - 1)];
                }
                if (child.isFocusable()) {
                    return child; // dijit._Widget
                }
                child = this._getSiblingOfChild(child, dir);
            }
            // no focusable child found
            return null; // dijit._Widget
        },

        _setIsExpandAttr: function (value) {
            this.toggleButtonState.set("checked", value);
        },

        isExpanded: function () {
            // summary:
            //    Return a flag indication the button state.
            //
            // tags:
            //    public

            return this.toggleButtonState.get("checked");
        },

        focusNext: function () {
            var lastFocusableMenuItem = this._getLastFocusableChild();
            if (this.toggleButtonState.isFocusable() && this.focusedChild === lastFocusableMenuItem) {
                return this.focusChild(this.toggleButtonState);
            }

            return this.inherited(arguments);
        },

        focusPrev: function () {
            var firstFocusableMenuItem = this._getFirstFocusableChild();
            if (this.toggleButtonState.isFocusable() && this.focusedChild === firstFocusableMenuItem) {
                return this.focusChild(this.toggleButtonState);
            }

            return this.inherited(arguments);
        },

        focusChild: function (/*dijit._Widget*/widget, /*Boolean*/last) {
            if (this.isExpanded()) {
                this.inherited(arguments);
            }
        },

        _toggleButtonStateClick: function (val) {
            // summary:
            //      Toggles view setting button, to show, or hide additional buttons.
            // val: boolean
            //      Flag to indicate whether the view setting is checked.
            // tags:
            //      protected

            this.focusedChild = null; // reset the last focus child, to force the menu navigation works correctly.

            val ? this._expand() : this._collapse();

            // trigger onChange callback
            this.onChange(val);
        },

        onChange: function (val) {
            // summary:
            //      Will be trigger whenever the view setting button is clicked.
            // tags:
            //      callback
        },

        _expand: function () {
            // summary:
            //      Expands the view setting button, to show additional buttons.
            // tags:
            //      private

            var children = this.containerNode.getChildren(),
                container = this.showInGroup ? this.getParent().domNode : this.itemsContainer,
                expandWidth = 6 + domGeometry.getMarginBox(container).w; // 6px padding from children offset for CSS animation.

            if (!this.hasToggleableChildren) {
                domClass.add(this.showInGroup ? this.getParent().domNode : this.domNode, "dijitChecked");
            }

            // Nodes that contain no children should do nothing animation-wise
            if (children.length === 0) { return; }

            array.forEach(children, function (item) {
                expandWidth += domGeometry.getMarginBox(item.domNode).w;
            }, this);

            domStyle.set(container, "maxWidth", expandWidth + 10 + "px"); // Add an additional 10px for the extra padding on the ul
            domClass.add(container, "epi-expandoButtonParentNodeOpen");
        },

        _collapse: function () {
            // summary:
            //      Collapses the view setting button, to hide additional buttons.
            // tags:
            //      private

            var container = this.showInGroup ? this.getParent().domNode : this.itemsContainer;

            if (!this.hasToggleableChildren) {
                domClass.remove(this.showInGroup ? this.getParent().domNode : this.domNode, "dijitChecked");
            }

            domStyle.set(container, "maxWidth", "");
            domClass.remove(container, "epi-expandoButtonParentNodeOpen");
        }
    });
});