﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-attr",
    "dojo/dom-class",
    "dojo/dom-style",

    "dojo/Evented",
    "dojo/keys",
// dijit
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/_WidgetsInTemplateMixin",

    "dijit/focus",
    "dijit/form/TextBox",
// dojox
    "dojox/html/entities",
// dgrid
    "dgrid/util/misc",
// epi
    "epi",
// Templates
    "dojo/text!./templates/SearchBox.html",
// Resources
    "epi/i18n!epi/shell/ui/nls/EPiServer.Shell.UI.Resources.SearchBox"
],
function (
// dojo
    declare,
    lang,

    domAttr,
    domClass,
    domStyle,

    Evented,
    keys,
// dijit
    _TemplatedMixin,
    _Widget,
    _WidgetsInTemplateMixin,

    focusManager,
    TextBox,
// dojox
    htmlEntities,
// dgrid
    miscUtil,
// epi
    epi,
// Templates
    template,
// Resources
    resources
) {
    return declare([TextBox, Evented], {
        // tags:
        //      internal

        templateString: template,

        // placeHolder: String
        //      Defines a hint to help users fill out the input field (as defined in HTML 5).
        //      This should only contain plain text (no html markup).
        placeHolder: epi.resources.action.search,

        // commonRes: [private] Object
        //    Common localization resources,
        commonRes: epi.resources,

        // defaultDelay: [Integer]
        //      Delay in milliseconds between when user types something and we start
        //      searching based on that value.
        // tags:
        //      protected
        defaultDelay: 500,

        // encodeSearchText: [public] Boolean
        //      Indicated that need to encode search text or not.
        encodeSearchText: false,

        postCreate: function () {
            // summary:
            //		Post widget creation. Initialize event for buttons
            // tags:
            //      Protected

            this.inherited(arguments);

            domAttr.set(this.clearButton, "title", resources.clear);

            // clear textbox value when clicking on clear button
            this.connect(this.clearButton, "onclick", lang.hitch(this, function () {
                this.set("value", "");
                this.focus();
                this.toggleClearButton();
                this._valueChanged();
            }));

            this.toggleClearButton();

            this.connect(this, "_onInput",
                miscUtil.debounce(lang.hitch(this, function (event) {
                    this.toggleClearButton();
                    this._valueChanged(event);
                }), null, this.defaultDelay));
        },

        toggleClearButton: function () {
            // summary:
            //      Set visibility for clear button
            // tags:
            //      private

            // show clear button only if textbox has value
            domStyle.set(this.clearButton, "visibility", this.textbox.value ? "visible" : "hidden");
        },

        focus: function () {
            // summary:
            //      Set focus for textbox
            // tags:
            //      public

            focusManager.focus(this.textbox);
        },

        _setValueAttr: function (value) {
            // summary:
            //      Set value for textbox
            // tags:
            //      private

            this.textbox.value = value;
        },

        _valueChanged: function (/* Event */event) {
            // summary:
            //      Trigger when textbox value has been changed
            // tags:
            //      private

            if (event && event.keyCode) {
                switch (event.keyCode) {
                    case keys.PAGE_DOWN:
                    case keys.DOWN_ARROW:
                    case keys.PAGE_UP:
                    case keys.UP_ARROW:
                    case keys.LEFT_ARROW:
                    case keys.RIGHT_ARROW:
                        // do nothing when navigating, selecting text
                        return;
                }
            }

            this.emit("searchBoxChange", this.encodeSearchText ? htmlEntities.encode(this.textbox.value) : this.textbox.value);
        }
    });
});
