define([
    "dojo/_base/json",
    "../widget/dialog/Alert",
    "../widget/dialog/ErrorDialog",
], function (json, Alert, ErrorDialog) {

    var _parseResponseText = function(responseData) {
        if (typeof responseData === "string") {
            try {
                return json.fromJson(responseData);
            } catch (e){
                console.warn("Failed to parse responseText as json", e);
            }
        }
        return null;
    };

    var handler = {

        forXhr: function (response) {
            // summary:
            //      Shows an alert dialog with the response message from a failed xhr result.
            //      If it's a 5xx error then the entire body is shown, otherwise the message
            //      is shown in an Alert dialog.
            //
            // tags: 
            //      private

            if (response.status >= 500) {
                ErrorDialog.showXmlHttpError(response, response);
            } else {
                var data = _parseResponseText(response.responseText) || {};
                var message = data.message || response.message;

                var dialog = new Alert({
                    description: message
                });

                dialog.show();
            }
        }
    };

    return handler;

});
