﻿define([
    "dojo/_base/declare",
    "dojox/html/entities",
    "epi",
    "epi/dependency"
],

function (
    declare,
    entities,
    epi,
    dependency
) {

    return {
        // tags:
        //      public

        toUserFriendlyString: function (/*string*/username, /*string*/currentUsername, /*boolean*/capitalizeUsername, /*boolean*/keepEmptyUsername) {
            // summary:
            //      Get friendly username: if the username to be displayed is the same
            //      as the current username, this will returns "you".
            // username:
            //    The username to be displayed.
            // currentUsername:
            //    The username to compare with (optional) to see if it's the current user.
            // capitalizeUsername:
            //    If the first character should be in upper case.
            // keepEmptyUsername:
            //    If the empty username should be kept or replaced by "Installer".
            // tags:
            //    public

            if (!currentUsername) {
                currentUsername = this._getCurrentUsername();
            }

            var emtyUsername = keepEmptyUsername ? "" : epi.resources.text.installer,
                friendlyUsername = username && currentUsername.toLowerCase() === username.toLowerCase() ? epi.resources.text.you : username || emtyUsername;
            if (capitalizeUsername) {
                friendlyUsername = friendlyUsername.charAt(0).toUpperCase() + friendlyUsername.slice(1);
            }
            return friendlyUsername;
        },

        toUserFriendlyHtml: function (/*string*/username, /*string*/currentUsername, /*boolean*/capitalizeUsername, /*boolean*/keepEmptyUsername) {
            // summary:
            //    Creates a friendly version of the username and wraps it inside a span tagged with a given class for styling purposes.
            //
            // username:
            //    The username to be displayed.
            // currentUsername:
            //    The username to compare with (optional) to see if it's the current user.
            // capitalizeUsername:
            //    If the first character should be in upper case.
            // keepEmptyUsername:
            //    If the empty username should be kept or replaced by "Installer".
            // tags:
            //    public

            var formattedUserName = this.toUserFriendlyString(username, currentUsername, capitalizeUsername, keepEmptyUsername);
            if (formattedUserName === "") {
                return formattedUserName;
            }
            return "<span class='epi-username'>" + entities.encode(formattedUserName) + "</span>";
        },

        _getCurrentUsername: function () {
            if (!this.currentUsername) {
                var profile = dependency.resolve("epi.shell.Profile");
		//TODO: Make this Async
                this.currentUsername = profile.get("userName");
            }
            return this.currentUsername;
        }
    };
});
