define(["build/buildControl", "build/fileUtils", "build/fs", "build/replace"], function (bc, fileUtils, fs, replace) {
    var getFiletype = fileUtils.getFiletype,
    	encodingMap =
    // map from file type to encoding
			(bc.transformConfig.read && bc.transformConfig.read.encoding) || {
			    css: "utf8",
			    html: "utf8",
			    htm: "utf8",
			    js: "utf8",
			    json: "utf8",
			    asc: "utf8",
			    c: "utf8",
			    cpp: "utf8",
			    log: "utf8",
			    conf: "utf8",
			    text: "utf8",
			    txt: "utf8",
			    dtd: "utf8",
			    xml: "utf8",
			    png: undefined,
			    jpg: undefined,
			    jpeg: undefined,
			    gif: undefined
			};

    return function (resource, callback) {
        resource.getText = function () {
            if (!this.replacementsApplied) {
                this.replacementsApplied = true;
                if (bc.replacements[this.src]) {
                    this.text = replace(this.text, bc.replacements[this.src]);
                }
            }
            return this.text;
        };

        resource.setText = function (text) {
            resource.text = text;
            resource.getText = function () { return this.text; };
            return text;
        };


        var filetype = getFiletype(resource.src, 1);
        // the expression is a little odd since undefined is a legitimate encodingMap value
        resource.encoding = resource.encoding || (!(filetype in encodingMap) && "utf8") || encodingMap[filetype];
        fs.readFile(resource.src, resource.encoding, function (err, data) {

            // It appears that java does not understand a BOM in UTF-8 encoded files, see http://bugs.sun.com/view_bug.do?bug_id=4508058
            // The UTF-8 BOM "0xEF 0xBB 0xBF" (which translates into UTF-16 0xFEFF / 65279), when treated as a regular character is
            // "ZERO WIDTH NO-BREAK SPACE". However, if this is the first character in the file, we assume it's a non-interpreted UTF-8 BOM.
            // Also see http://tools.ietf.org/html/rfc3629#section-6 for details on the UTF-8 BOM.

            if (data.charCodeAt(0) == 65279) {
                data = data.substring(1);
            }

            if (!err) {
                resource.text = data;
            }
            callback(resource, err);
        });
        return callback;
    };
});
